/****************************************************************************
** File name    :  HTMemCacheData.cpp
** Description  :  define memory data struct
** Create date  :  2018.09.01
** Auther by    :  Liuyx
** Version info :  V1.0.01
** Copyrigth By:  xi'an huatek, Inc Co., Ltd
** Update record:
** DATE         AUTHER   DESC
** -------------------------------------------------------------------------
** 2018.09.01   Liuyx    first build
****************************************************************************/
#include "HTGlobal.h"
#include "HTIEC104.h"
#include "HTMemCacheData.h"

static const char *_FILE_ = "HTMemData.cpp";


/***************************************************************************
** function name :  vLoadBYQThresholdTable
** deacription   :  get byq define value & site_id-->site_code relation.
** parameter     :  none
** return code   :  NULL
***************************************************************************/
static void vLoadBYQThresholdTable(bool is_load)
{
    char szSql[1024] = { 0 };
    int  wstate = 0;
    CDBMySQL *pdbHandle = CDBMySQL::Instance();
    if (!pdbHandle) return;

    snprintf(szSql, sizeof(szSql), "select t1.name,t1.site_id,t1.system_code,t.main_tree_id,t.work_status,"
        "t.coolingwater_inlet_pressure, t.coolingwater_outlet_pressure, t.coolingwater_inlet_temperature, t.coolingwater_outlet_temperature,"
        "t.circulating_oil_inlet_pressure,t.circulating_oil_outlet_pressure, t.winding_temperature, t.oilpillow_oillevel, t.transformer_Ontology_oillevel,"
        "t.oil_temperature, t.top_oil_temperature,t.circulating_oil_inlet_temperature,t.circulating_oil_out_temperature "
        "from busi_site_tree t1, transformer_runing_status_threshold t "
        "where t1.id = t.main_tree_id and t1.status = '1' and t1.site_id = t.site_id "
        "order BY t1.system_code");

    MYSQL_RES *res = pdbHandle->SelectRecord(szSql);
    if (!res)  return;

    MYSQL_ROW row = NULL;
    while (row = pdbHandle->GetRecord(res))
    {
        ST_BYQ_RUN_STATE_THRESHOLD stByq;
        int idx = 0;
        ST_STAT_THRESHOLD stThreshold;
        string strKey = "";
        memset(&stByq, 0x00, sizeof(ST_BYQ_RUN_STATE_THRESHOLD));
        memset(&stThreshold, 0x00, sizeof(ST_STAT_THRESHOLD));

        if (row[0] && strlen(row[0]) > 0) {
            strncpy((char*)stByq.dev_name, (const char*)row[0], sizeof(stByq.dev_name));
        }

        if (row[1] && strlen(row[1]) > 0) {
            strncpy((char*)stByq.site_id, (const char*)row[1], sizeof(stByq.site_id));
        }

        if (row[2] && strlen(row[2]) > 0) {
            strncpy((char*)stByq.system_code, (const char*)row[2], sizeof(stByq.system_code));
            //strKey = (char*)stByq.system_code;
        }
        if (row[3] && strlen(row[3]) > 0) {
            strncpy((char*)stByq.eqm_code, (const char*)row[3], sizeof(stByq.eqm_code)); // main_tree_id源自表busi_site_tree.id
            strKey = (char*)stByq.eqm_code;
        }

        //工作状态：1 - 发电状态 2：抽水状态 3：空闲状态
        if (row[4] && strlen(row[4]) > 0) {
            wstate = atoi((const char*)row[4]);
        }

        //stThreshold.wstate = wstate;
        if (row[5] && strlen(row[5]) > 0) {
            strcpy(stThreshold.stField[idx].field, "coolingwater_inlet_pressure");
            stThreshold.stField[idx++].value = atof((const char*)row[5]); // 冷却水进口压力
        }
        else {
            strcpy(stThreshold.stField[idx].field, "coolingwater_inlet_pressure");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 冷却水进口压力
        }

        if (row[6] && strlen(row[6]) > 0) {
            strcpy(stThreshold.stField[idx].field, "coolingwater_outlet_pressure");
            stThreshold.stField[idx++].value = atof((const char*)row[6]); // 冷却水出口压力
        }
        else {
            strcpy(stThreshold.stField[idx].field, "coolingwater_outlet_pressure");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 冷却水出口压力
        }
        if (row[7] && strlen(row[7]) > 0) {
            strcpy(stThreshold.stField[idx].field, "coolingwater_inlet_temperature");
            stThreshold.stField[idx++].value = atof((const char*)row[7]); // 冷却水进口温度
        }
        else {
            strcpy(stThreshold.stField[idx].field, "coolingwater_inlet_temperature");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 冷却水进口温度
        }
        if (row[8] && strlen(row[8]) > 0) {
            strcpy(stThreshold.stField[idx].field, "coolingwater_outlet_temperature");
            stThreshold.stField[idx++].value = atof((const char*)row[8]); // 冷却水出口温度
        }
        else {
            strcpy(stThreshold.stField[idx].field, "coolingwater_outlet_temperature");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 冷却水出口温度
        }
        if (row[9] && strlen(row[9]) > 0) {
            strcpy(stThreshold.stField[idx].field, "circulating_oil_inlet_pressure");
            stThreshold.stField[idx++].value = atof((const char*)row[9]); // 循环油进口压力
        }
        else {
            strcpy(stThreshold.stField[idx].field, "circulating_oil_inlet_pressure");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 循环油进口压力
        }
        if (row[10] && strlen(row[10]) > 0) {
            strcpy(stThreshold.stField[idx].field, "circulating_oil_outlet_pressure");
            stThreshold.stField[idx++].value = atof((const char*)row[10]); // 循环油出口压力
        }
        else {
            strcpy(stThreshold.stField[idx].field, "circulating_oil_outlet_pressure");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 循环油出口压力
        }
        if (row[11] && strlen(row[11]) > 0) {
            strcpy(stThreshold.stField[idx].field, "winding_temperature");
            stThreshold.stField[idx++].value = atof((const char*)row[11]); // 绕组温度
        }
        else {
            strcpy(stThreshold.stField[idx].field, "winding_temperature");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 绕组温度
        }
        if (row[12] && strlen(row[12]) > 0) {
            strcpy(stThreshold.stField[idx].field, "oilpillow_oillevel");
            stThreshold.stField[idx++].value = atof((const char*)row[12]); // 油枕油位
        }
        else {
            strcpy(stThreshold.stField[idx].field, "oilpillow_oillevel");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 油枕油位
        }
        if (row[13] && strlen(row[13]) > 0) {
            strcpy(stThreshold.stField[idx].field, "transformer_Ontology_oillevel");
            stThreshold.stField[idx++].value = atof((const char*)row[13]); // 变压器本体油位
        }
        else {
            strcpy(stThreshold.stField[idx].field, "transformer_Ontology_oillevel");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 变压器本体油位
        }
        if (row[14] && strlen(row[14]) > 0) {
            strcpy(stThreshold.stField[idx].field, "oil_temperature");
            stThreshold.stField[idx++].value = atof((const char*)row[14]); // 变压器本体油温
        }
        else {
            strcpy(stThreshold.stField[idx].field, "oil_temperature");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 变压器本体油温
        }
        if (row[15] && strlen(row[15]) > 0) {
            strcpy(stThreshold.stField[idx].field, "top_oil_temperature");
            stThreshold.stField[idx++].value = atof((const char*)row[15]); // 顶层油温
        }
        else {
            strcpy(stThreshold.stField[idx].field, "top_oil_temperature");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 顶层油温
        }
        if (row[16] && strlen(row[16]) > 0) {
            strcpy(stThreshold.stField[idx].field, "circulating_oil_inlet_temperature");
            stThreshold.stField[idx++].value = atof((const char*)row[16]); // 变压器油进口油温度(℃)
        }
        else {
            strcpy(stThreshold.stField[idx].field, "circulating_oil_inlet_temperature");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 变压器油进口油温度(℃)
        }
        if (row[17] && strlen(row[17]) > 0) {
            strcpy(stThreshold.stField[idx].field, "circulating_oil_out_temperature");
            stThreshold.stField[idx++].value = atof((const char*)row[17]); // 变压器油出口油温度(℃)
        }
        else {
            strcpy(stThreshold.stField[idx].field, "circulating_oil_out_temperature");
            stThreshold.stField[idx++].value = HT_INVALID_VALUE; // 变压器油出口油温度(℃)
        }
        stThreshold.fnum = idx;
        if (idx >= MAX_RUN_STATUS_COUNT) {
            vPrtLogMsg(LOG_WARNG, RET_OK, "Onec devices max threshold count:%d, current count:%d", MAX_RUN_STATUS_COUNT,idx);
            stThreshold.fnum = MAX_RUN_STATUS_COUNT - 1;
        }

        map<string, ST_BYQ_RUN_STATE_THRESHOLD>::iterator m_pIter;
        mutex_lock(g_map_byq_mutex);
        m_pIter = g_map_thres_byq.find((char*)stByq.eqm_code);
        if (m_pIter != g_map_thres_byq.end())
        {
            switch (wstate)
            {
            case 1:
                memcpy(&(m_pIter->second).stThreashod[0], &stThreshold, sizeof(ST_STAT_THRESHOLD));
                break;
            case 2:
                memcpy(&(m_pIter->second).stThreashod[1], &stThreshold, sizeof(ST_STAT_THRESHOLD));
                break;
            case 3:
                memcpy(&(m_pIter->second).stThreashod[2], &stThreshold, sizeof(ST_STAT_THRESHOLD));
                break;
            default:;
            }
        }
        else {
            switch (wstate)
            {
            case 1:
                memcpy(&stByq.stThreashod[0], &stThreshold, sizeof(ST_STAT_THRESHOLD));
                break;
            case 2:
                memcpy(&stByq.stThreashod[1], &stThreshold, sizeof(ST_STAT_THRESHOLD));
                break;
            case 3:
                memcpy(&stByq.stThreashod[2], &stThreshold, sizeof(ST_STAT_THRESHOLD));
                break;
            default:;
            }
            g_map_thres_byq.insert(map<string, ST_BYQ_RUN_STATE_THRESHOLD>::value_type(strKey, stByq));
        }
        mutex_unlock(g_map_byq_mutex);
    }
    pdbHandle->FreeRecord(res);
    return;
}

static void vPrtBYQThresholdTable(void)
{
    map<string, ST_BYQ_RUN_STATE_THRESHOLD>::iterator m_pIter;
    m_pIter = g_map_thres_byq.begin();
    vPrtLogMsg(LOG_DEBUG, RET_OK, "---------Print BYQ Threshold Tables----------");
    vPrtLogMsg(LOG_DEBUG, RET_OK, "site_id\t\t\t\t eqm_code\t\t\t system_code\t ST  dev_name");
    while (m_pIter != g_map_thres_byq.end())
    {
        vPrtLogMsg(LOG_DEBUG, RET_OK, "%-s %s %s %c %s",
            (m_pIter->second).site_id, (m_pIter->second).eqm_code, (m_pIter->second).system_code, 
            ((m_pIter->second).wstate == 0 ? 0x2f : 0x30+(m_pIter->second).wstate), (m_pIter->second).dev_name);
        m_pIter++;
    }
    vPrtLogMsg(LOG_DEBUG, RET_OK, "---------- BYQ_Threshold_size:%d-----------", g_map_thres_byq.size());
    return;
}

// 根据运行工况的字典名称，获取阈值
double getThresholdValuse(const char *eqm_code, const char *pFiledName, int jobState)
{
    ST_BYQ_RUN_STATE_THRESHOLD stByq;
    map<string, ST_BYQ_RUN_STATE_THRESHOLD>::iterator m_pIter;
    double val = 0.0f;
    if (jobState <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "Check Device Statue:%d(invalid)", jobState);
        return val;
    }
    mutex_lock(g_map_byq_mutex);
    m_pIter = g_map_thres_byq.find((char*)eqm_code);
    if (m_pIter == g_map_thres_byq.end())
    {
        mutex_unlock(g_map_byq_mutex);
        return val;
    }
    memcpy(&stByq, &m_pIter->second, sizeof(ST_BYQ_RUN_STATE_THRESHOLD));
    mutex_unlock(g_map_byq_mutex);
    
    int i = 0;    
    while (i < stByq.stThreashod[jobState-1].fnum)
    {
        if (stringncasecmp(  (const char*)stByq.stThreashod[jobState-1].stField[i].field, (const char*)pFiledName, strlen((const char*)pFiledName)) == 0)
        {
            return stByq.stThreashod[jobState - 1].stField[i].value == HT_INVALID_VALUE ? 0 : stByq.stThreashod[jobState - 1].stField[i].value;
        }
        i++;
    }
    return val;
}

//加载IEC104点表数据
static void vLoadIECPointTable()
{
    char szSql[512] = { 0 };
    
    CDBMySQL *pdbHandle = CDBMySQL::Instance();
    if (!pdbHandle) return;
    snprintf(szSql, sizeof(szSql), "select p.site_id, p.site_tree_id, p.stype, p.sadr, p.sno,p.isget,p.unit,"
        "p.sysunit,p.devrate,p.thresholdfield,p.desct,"
        "SUBSTR(t.equipment_type from 1 for 1) as eqm_type,p.warnflag,t.system_code "
        "from iec_points_conf p, busi_site_tree t "
        "where p.sadr > 0 and p.state ='1' and t.id=p.site_tree_id and t.status='1' "
        "ORDER BY p.site_id, p.site_tree_id, p.sadr");
    MYSQL_RES *res = pdbHandle->SelectRecord(szSql);
    if (!res)  return;

    MYSQL_ROW row = NULL;
    while (row = pdbHandle->GetRecord(res))
    {
        ST_IECPOINT_TABLE stPoint;
        ST_DEVICE_TIME_STAT stDevStat;
        memset(&stPoint, 0x00, sizeof(ST_IECPOINT_TABLE));
        memset(&stDevStat, 0x00, sizeof(ST_DEVICE_TIME_STAT));
        if (row[0] && strlen(row[0]) > 0)
        {
            strncpy((char*)stPoint.site_id, (const char*)row[0], sizeof(stPoint.site_id));
        }
        if (row[1] && strlen(row[1]) > 0) {
            strncpy((char*)stPoint.eqm_code, (const char*)row[1], sizeof(stPoint.eqm_code));
        }

        if (row[2] && strlen(row[2]) > 0) {
            stPoint.stype = atoi(row[2]);
        }
        if (row[3] && strlen(row[3]) > 0) {
            stPoint.sadr = atoi(row[3]);
        }
        if (row[4] && strlen(row[4]) > 0) {
            stPoint.sno = atoi(row[4]);
        }
        if (row[5] && strlen(row[5]) > 0) {
            stPoint.isget = (unsigned char)atoi(row[5]);
        }
        if (row[6] && strlen(row[6]) > 0) {
            stPoint.unit = (unsigned short)atoi(row[6]);
        }
        if (row[7] && strlen(row[7]) > 0) {
            stPoint.sysunit = (unsigned short)atoi(row[7]);
        }
        if (row[8] && strlen(row[8]) > 0) {
            stPoint.devrate = atof(row[8]);
        }
        if (row[9] && strlen(row[9]) > 0) {
            strncpy((char*)stPoint.fieldname, row[9], sizeof(stPoint.fieldname));
        }
        if (row[10] && strlen(row[10]) > 0) {
            strncpy((char*)stPoint.poidesc, row[10], sizeof(stPoint.poidesc));
            strncpy((char*)stDevStat.desct, row[10], sizeof(stDevStat.desct));
        }
        if (row[11] && strlen(row[11]) > 0) {
            stPoint.eqm_type = (unsigned char)atoi(row[11]);
        }
        if (row[12] && strlen(row[12]) > 0) {
            stPoint.warnFlag = (unsigned char)atoi(row[12]);
        }
        if (row[13] && strlen(row[13]) > 0) {
            strncpy((char*)stPoint.sys_code, (const char*)row[13], sizeof(stPoint.sys_code));
            strncpy((char*)stDevStat.sys_code, (const char*)row[13], sizeof(stDevStat.sys_code));
        }
        mutex_lock(g_map_iec_mutex);
        map<unsigned int, ST_IECPOINT_TABLE>::iterator m_pIter;
        m_pIter = g_map_iec.find(stPoint.sadr);
        if (m_pIter == g_map_iec.end())
        {
            stPoint.bfault = 0;   // 初始状态0,无事故
            stPoint.wstate = 3;   // 默认为空闲状态
            g_map_iec.insert(map<unsigned int, ST_IECPOINT_TABLE>::value_type(stPoint.sadr, stPoint));
        }
        else {
            // 保留原数据
            stPoint.bfault = (m_pIter->second).bfault;
            stPoint.wstate = (((m_pIter->second).wstate) & 0xff);
            stPoint.cval = (((m_pIter->second).cval) & 0xff);
            stPoint.fval = (m_pIter->second).fval;
            stPoint.dtime = (m_pIter->second).dtime;            
            //覆盖更新数据
            memcpy(&(m_pIter->second), &stPoint, sizeof(ST_IECPOINT_TABLE));
        }
        mutex_unlock(g_map_iec_mutex);
        if (stPoint.eqm_type == 1) {
            mutex_lock(g_map_dev_time_stat_mutex);
            map<string, ST_DEVICE_TIME_STAT>::iterator m_pIters;
            m_pIters = g_map_dev_time_stat.find((char*)stDevStat.sys_code);
            if (m_pIters == g_map_dev_time_stat.end())
            {
                stDevStat.usTime = time(NULL);   // 初始时间
                stDevStat.wstate = 3;   // 默认为空闲状态
                g_map_dev_time_stat.insert(map<string, ST_DEVICE_TIME_STAT>::value_type((char*)stDevStat.sys_code, stDevStat));
            }
            mutex_unlock(g_map_dev_time_stat_mutex);
        }
    }
    pdbHandle->FreeRecord(res);
    return;
}

static void vLoadIECPointTableNew()
{
	char szSql[512] = { 0 };

	CDBMySQL *pdbHandle = CDBMySQL::Instance();
	if (!pdbHandle) return;
	strncpy(szSql, "SELECT t1.sadr,t1.stype,t1.site_id,t1.sensor_id,t2.table_name,t1.field,t2.sensor_code,t2.system_code,t2.dtime_field_name,t2.devid_field_name,t2.one_dtime"
		" FROM niec_points AS t1 LEFT JOIN niec_sensors AS t2 ON t1.sensor_id=t2.`id`"
		" WHERE t1.sadr>0 AND t1.`status`=1 AND t2.`status`=1 "
		" ORDER BY t1.site_id,t1.sadr", sizeof(szSql));
	MYSQL_RES *res = pdbHandle->SelectRecord(szSql);
	if (!res)  return;

	MYSQL_ROW row = NULL;
	IEC_POINT stPoint;
	IEC_DEVICE stSensor;
	size_t idx = 0;
	map<unsigned int, CACHED_DEV_DATA>::iterator itCachedDev;
	map<unsigned int, IEC_DEVICE>::iterator itDev;
	while (row = pdbHandle->GetRecord(res))
	{
		memset(&stPoint, 0x00, sizeof(IEC_POINT));
		memset(&stSensor, 0x00, sizeof(IEC_DEVICE));
		// memset(&stDevStat, 0x00, sizeof(ST_DEVICE_TIME_STAT));

		idx = 0;
		if (row[idx] && strlen(row[idx]) > 0) {
			stPoint.sadr = strtoul(row[idx], NULL, 10);
		}
		idx++;
		if (row[idx] && strlen(row[idx]) > 0) {
			stPoint.stype = strtoul(row[idx], NULL, 10);
		}
		idx++;
		if (row[idx] && strlen(row[idx]) > 0)
		{
			stSensor.site_id = strtoul(row[idx], NULL, 10);
		}
		idx++;
		if (row[idx] && strlen(row[idx]) > 0)
		{
			stPoint.sensor_id = strtoul(row[idx], NULL, 10);
		}
		idx++;
		if (row[idx] && strlen(row[idx]) > 0) {
			strncpy((char*)stSensor.tableName, (const char*)row[idx], sizeof(stSensor.tableName));
		}
		idx++;
		if (row[idx] && strlen(row[idx]) > 0) {
			strncpy((char*)stPoint.fieldName, (const char*)row[idx], sizeof(stPoint.fieldName));
		}
		// t2.sensor_code,t2.system_code,t2.dtime_field_name,t2.devid_field_name,t2.one_dtime
		idx++;
		idx++;
		if (row[idx] && strlen(row[idx]) > 0) {
			strncpy((char*)stSensor.sys_code, (const char*)row[idx], sizeof(stSensor.sys_code));
		}
		idx++;
		if (row[idx] && strlen(row[idx]) > 0) {
			strncpy((char*)stSensor.dtimeFieldName, (const char*)row[idx], sizeof(stSensor.dtimeFieldName));
		}
		idx++;
		if (row[idx] && strlen(row[idx]) > 0) {
			strncpy((char*)stSensor.devidFieldName, (const char*)row[idx], sizeof(stSensor.devidFieldName));
		}
		idx++;
		if (row[idx] && strlen(row[idx]) > 0) {
			stSensor.one_dtime = (unsigned char)strtoul(row[idx], NULL, 10);
		}
		stSensor.sensor_id = stPoint.sensor_id;
		if (stPoint.stype == 1)	// YX
		{
			continue;
		}
		// vPrtLogMsg(LOG_WARNG, RET_OK, "----TABLE %s oneTime=%u", (const char*)stSensor.tableName, (unsigned int)stSensor.one_dtime);

		mutex_lock(g_map_iec_mutex_new);
		
		itDev = g_map_devices.find(stSensor.sensor_id);
		if (itDev == g_map_devices.end())
		{
			itDev = g_map_devices.insert(g_map_devices.end(), std::make_pair<>(stSensor.sensor_id, stSensor));
		}
		g_map_devices[stSensor.sensor_id] = stSensor;
		g_map_iec_new[stPoint.sadr] = stPoint;
		itCachedDev = g_map_dev_data.find(stSensor.sensor_id);
		if (itCachedDev == g_map_dev_data.end())
		{
			CACHED_DEV_DATA cachedData;
			cachedData.device = &(itDev->second);
			cachedData.firstTs = 0;
			cachedData.assignedFields = 0;
			itCachedDev = g_map_dev_data.insert(g_map_dev_data.end(), std::make_pair<>(stSensor.sensor_id, cachedData));
		}

		IEC_FIELD stField;
		stField.name = (const char*)stPoint.fieldName;
		stField.ts = 0;
		stField.fval = 0.0;
		itCachedDev->second.fields[stPoint.sadr] = std::make_pair<>(stField, false);
		mutex_unlock(g_map_iec_mutex_new);
	}
	pdbHandle->FreeRecord(res);
}

void ResetAllCachedDeviceData()
{
	map<unsigned int, CACHED_DEV_DATA>::iterator it;
	std::map<unsigned int, std::pair<IEC_FIELD, bool> >::iterator itField;

	mutex_lock(g_map_iec_mutex_new);
	for (it = g_map_dev_data.begin(); it != g_map_dev_data.end(); ++it)
	{
		ResetCachedDeviceData(it->second);
	}
	mutex_unlock(g_map_iec_mutex_new);
}

void ResetCachedDeviceData(CACHED_DEV_DATA& cachedDev)
{
	if (cachedDev.assignedFields == 0)
	{
		return;
	}
	
	cachedDev.firstTs = 0;

	std::map<unsigned int, std::pair<IEC_FIELD, bool> >::iterator itField;
	for (itField = cachedDev.fields.begin(); itField != cachedDev.fields.end(); ++itField)
	{
		itField->second.first.ts = 0;
		itField->second.first.fval = 0.0;
		itField->second.second = false;
	}
	
	cachedDev.assignedFields = 0;
}

std::string BuildSqlForDeviceData(CACHED_DEV_DATA& cachedDev)
{
	std::string fields, fieldValues, sql;
	char dataBuf[32] = { 0 };
#if __cplusplus < 201103L
	std::map<unsigned int, std::pair<IEC_FIELD, bool> >::iterator it;

	for (it = cachedDev.fields.begin(); it != cachedDev.fields.end(); ++it)
#else
	std::map<unsigned int, std::pair<IEC_FIELD, bool> >::const_iterator it;

	for (it = cachedDev.fields.cbegin(); it != cachedDev.fields.cend(); ++it)
#endif
	{
		fields.append(it->second.first.name);
		fields.append(",");

		fieldValues.append("'");
		snprintf(dataBuf, sizeof(dataBuf), "%0.4f", it->second.first.fval);
		fieldValues.append(dataBuf);
		fieldValues.append("',");

		if ((cachedDev.device->one_dtime == 0))
		{
			fields.append(it->second.first.name);
			fields.append("_time,");

			fieldValues.append("FROM_UNIXTIME(");
			snprintf(dataBuf, sizeof(dataBuf), "%lld", (long long)it->second.first.ts);
			fieldValues.append(dataBuf);
			fieldValues.append("),");
		}
	}

	sql = "INSERT INTO ";
	sql.append((const char *)cachedDev.device->tableName);
	sql.append("(");
	sql.append((const char *)cachedDev.device->devidFieldName);
	sql.append(",");
	if (cachedDev.device->one_dtime != 0)
	{
		sql.append((const char *)cachedDev.device->dtimeFieldName);
		sql.append(",");
	}
	sql.append(fields, 0, fields.size() - 1);

	sql.append(") VALUES(");

	snprintf(dataBuf, sizeof(dataBuf), "%u", cachedDev.device->sensor_id);
	sql.append(dataBuf);
	sql.append(",");
	if (cachedDev.device->one_dtime != 0)
	{
		sql.append("FROM_UNIXTIME(");
		snprintf(dataBuf, sizeof(dataBuf), "%lld", (long long)cachedDev.firstTs);
		sql.append(dataBuf);
		sql.append("),");
	}

	sql.append(fieldValues, 0, fieldValues.size() - 1);
	sql.append(")");

	return sql;
}

bool AssignValueToDeviceData(CACHED_DEV_DATA& cachedDev, const IEC_OBJVAL_NEW& val)
{
	std::map<unsigned int, std::pair<IEC_FIELD, bool> >::iterator it = cachedDev.fields.find(val.sadr);
	if (it != cachedDev.fields.end())
	{
		if (!(it->second.second))
		{
			if (cachedDev.firstTs == 0)
			{
				cachedDev.firstTs = val.dtime;
			}
			cachedDev.assignedFields++;
			it->second.first.fval = val.fval;
			it->second.first.ts = val.dtime;

			return true;
		}
	}

	return false;
}

// 刷新设备更新时间
void SethDevTimeStat(unsigned char *sys_code, unsigned char ws)
{
    mutex_lock(g_map_dev_time_stat_mutex);
    map<string, ST_DEVICE_TIME_STAT>::iterator m_pIters;
    m_pIters = g_map_dev_time_stat.find((char*)sys_code);
    if (m_pIters != g_map_dev_time_stat.end())
    {
        //(m_pIters->second).usTime = time(NULL);
        (m_pIters->second).wstate = ws;   // 默认为空闲状态        
    }
    mutex_unlock(g_map_dev_time_stat_mutex);
}
// 获取状态时间
time_t GetDevTimeStat(unsigned char *sys_code, unsigned char *ws)
{
    time_t usTime = 0;
    mutex_lock(g_map_dev_time_stat_mutex);
    map<string, ST_DEVICE_TIME_STAT>::iterator m_pIters;
    m_pIters = g_map_dev_time_stat.find((char*)sys_code);
    if (m_pIters != g_map_dev_time_stat.end())
    {
        if ((m_pIters->second).usTime == 0) {
            (m_pIters->second).usTime = time(NULL);   // 初始时间
        }
        else {
            usTime = (m_pIters->second).usTime;
            (m_pIters->second).usTime = time(NULL)+1;
        }
        if ((m_pIters->second).wstate == 0) {
            (m_pIters->second).wstate = 3;  // 默认为空闲状态
        }
        *ws = (m_pIters->second).wstate ;           
    }
    mutex_unlock(g_map_dev_time_stat_mutex);
    return usTime;
}

// 打印时间
void vPrtTimeStat()
{
    char sTime[32] = { 0 };
    map<string, ST_DEVICE_TIME_STAT>::iterator m_pIter;
    mutex_lock(g_map_dev_time_stat_mutex);
    m_pIter = g_map_dev_time_stat.begin();
    vPrtLogMsg(LOG_WARNG, RET_OK, "---------Print time and state Table Config----------");
    vPrtLogMsg(LOG_WARNG, RET_OK, "      sys_code\t\t\t time \t\t stat \t desc");
    while (m_pIter != g_map_dev_time_stat.end())
    {
        vTranHostTimeFmt((m_pIter->second).usTime, sTime);
        vPrtLogMsg(LOG_WARNG, RET_OK, "--> %s    %s    %c     %s", (m_pIter->second).sys_code,
            sTime, ((m_pIter->second).wstate + 0x30), (m_pIter->second).desct);
        m_pIter++;
    }
    vPrtLogMsg(LOG_WARNG, RET_OK, "---------- Point time and state :%d-----------", g_map_dev_time_stat.size());
    mutex_unlock(g_map_dev_time_stat_mutex);
    return;

}
// 打印点表配置信息
static void vPrtIECPointTable()
{
    map<unsigned int, ST_IECPOINT_TABLE>::iterator m_pIter;
    mutex_lock(g_map_iec_mutex);
    m_pIter = g_map_iec.begin();
    vPrtLogMsg(LOG_WARNG, RET_OK, "---------Print IEC104 Point Table Config Relation----------");
    vPrtLogMsg(LOG_WARNG, RET_OK, "sys_code\t\t\t\t\tsadr\t\t  value  G  rate(%%)  EQM_T   desc");
    while (m_pIter != g_map_iec.end())
    {
        //vPrtLogMsg(LOG_WARNG, RET_OK, "--> %s  %s  %05d(0x%04x)  %03d  %.4f  %c  %.2f     %d   %s", (m_pIter->second).sys_code,
        vPrtLogMsg(LOG_WARNG, RET_OK, "--> %s  %s  %05d(0x%04x)  %.4f  %c  %.2f     %d   %s", (m_pIter->second).sys_code,
            ((m_pIter->second).stype == 1 ? "YX" : "YC"),
            (m_pIter->second).sadr, (m_pIter->second).sadr,
            // (m_pIter->second).sno, 
            ((m_pIter->second).stype == 1) ? ((m_pIter->second).cval & 0xff) : (m_pIter->second).fval,
            ((m_pIter->second).isget == 0 ? 0x2f : (m_pIter->second).isget+0x30), (m_pIter->second).devrate, 
            (m_pIter->second).eqm_type,  (m_pIter->second).poidesc);
        m_pIter++;
    }
    vPrtLogMsg(LOG_WARNG, RET_OK, "---------- Point Table_size:%d-----------", g_map_iec.size());
    mutex_unlock(g_map_iec_mutex);
    return;
}

//加载主变设备ID列表,用于缓存主变设备工作状态,1:发电 2:抽水 3:空闲
static void vLoadByqDeviceStateTable()
{
    char szSql[512] = { 0 };
    CDBMySQL *pdbHandle = CDBMySQL::Instance();
    if (!pdbHandle) return;
    snprintf(szSql, sizeof(szSql), "select site_id,id,name from busi_site_tree t where equipment_type='111' and status='1' ORDER BY name"); 
    MYSQL_RES *res = pdbHandle->SelectRecord(szSql);
    if (!res)  return;

    string strKey = "";
    MYSQL_ROW row = NULL;
    map<string, ST_BYQWORK_STATUS>::iterator m_pIter;
    while (row = pdbHandle->GetRecord(res))
    {
        ST_BYQWORK_STATUS stWstate;
        memset(&stWstate, 0x00, sizeof(ST_BYQWORK_STATUS));
        if (row[0] && strlen(row[0]) > 0)
        {
            strncpy((char*)stWstate.site_id, (const char*)row[0], sizeof(stWstate.site_id));
        }
        if (row[1] && strlen(row[1]) > 0)
        {
            strncpy((char*)stWstate.eqm_code, (const char*)row[1], sizeof(stWstate.eqm_code));
            strKey = row[1];
        }
        if (row[2] && strlen(row[2]) > 0) {
            strncpy((char*)stWstate.eqm_name, (const char*)row[2], sizeof(stWstate.eqm_name));
        }
        mutex_lock(g_map_work_mutex);
        bool isAdd = false;
        m_pIter = g_map_work.find(strKey);
        if (m_pIter != g_map_work.end())
        {
            strcpy((char*)(m_pIter->second).site_id, (const char*)stWstate.site_id);
            strcpy((char*)(m_pIter->second).eqm_code, (const char*)stWstate.eqm_code);
            strcpy((char*)(m_pIter->second).eqm_name, (const char*)stWstate.eqm_name);
        }
        else{
            stWstate.bfault = 0;
            stWstate.wstate = 3;
            g_map_work.insert(map<string, ST_BYQWORK_STATUS>::value_type(strKey, stWstate));
        }
        mutex_unlock(g_map_work_mutex);
    }
    pdbHandle->FreeRecord(res);
    return;
}
// 打印变压器工作状态
static void vPrtByqDeviceStateTable()
{
    int i = 0;
    map<string, ST_BYQWORK_STATUS>::iterator m_pIter;
    mutex_lock(g_map_work_mutex);
    vPrtLogMsg(LOG_DEBUG, RET_OK, "---------Print byq work state Table ----------");
    vPrtLogMsg(LOG_DEBUG, RET_OK, " index site_id\t\t\t\t eqm_code\t\t\t wstate\t fault\teqm_name");
    m_pIter = g_map_work.begin();
    while (m_pIter != g_map_work.end())
    {
        vPrtLogMsg(LOG_DEBUG, RET_OK, "--> %d  %s %s  %d    %s     %s", i + 1,
            (m_pIter->second).site_id,
            (m_pIter->second).eqm_code,
            (m_pIter->second).wstate,
            ((m_pIter->second).bfault == 0 ? "Normal" : "Fault"),
            (m_pIter->second).eqm_name);  //0:分 1 : 合
        m_pIter++; i++;
    }
    vPrtLogMsg(LOG_DEBUG, RET_OK, "---------- Print byq work state, size:%d-----------", g_map_work.size());
    mutex_unlock(g_map_work_mutex);
    return;
}
// 设置主变工作状态
void vSetByqDeviceState(unsigned char *pszEqm_code, unsigned char state)
{
    map<string, ST_BYQWORK_STATUS>::iterator m_pIter;
    mutex_lock(g_map_work_mutex);
    m_pIter = g_map_work.find((char*)pszEqm_code);
    if (m_pIter != g_map_work.end())
    {
        (m_pIter->second).wstate = state;
    }
    mutex_unlock(g_map_work_mutex);
}

// 获取工作状态
unsigned char cGetByqDeviceState(unsigned char *pszEqm_code)
{
    unsigned char state = 3;
    map<string, ST_BYQWORK_STATUS>::iterator m_pIter;
    mutex_lock(g_map_work_mutex);
    m_pIter = g_map_work.find((char*)pszEqm_code);
    if (m_pIter != g_map_work.end())
    {
        state = ((m_pIter->second).wstate & 0xff);
    }
    mutex_unlock(g_map_work_mutex);
    return state;
}

// 设置主变事故状态
void vSetByqDeviceFaultState(unsigned char *pszEqm_code, unsigned char state)
{
    map<string, ST_BYQWORK_STATUS>::iterator m_pIter;
    mutex_lock(g_map_work_mutex);
    m_pIter = g_map_work.find((char*)pszEqm_code);
    if (m_pIter != g_map_work.end())
    {
        (m_pIter->second).bfault = state;
    }
    mutex_unlock(g_map_work_mutex);
}

// 获取主变事故状态
unsigned char cGetByqDeviceFaultState(unsigned char *pszEqm_code)
{
    unsigned char state = 3;
    map<string, ST_BYQWORK_STATUS>::iterator m_pIter;
    mutex_lock(g_map_work_mutex);
    m_pIter = g_map_work.find((char*)pszEqm_code);
    if (m_pIter != g_map_work.end())
    {
        state = ((m_pIter->second).bfault & 0xff);
    }
    mutex_unlock(g_map_work_mutex);
    return state;
}

//加载设备关系表
static void vLoadDeviceRelationsTable()
{
    char szSql[512] = { 0 };
    CDBMySQL *pdbHandle = CDBMySQL::Instance();
    if (!pdbHandle) return;
    snprintf(szSql, sizeof(szSql), "select t.site_id,b.transformer_id, b.equipment_id,SUBSTR(t.equipment_type from 1 for 1) as eqm_type, "
        "t.system_code "
        "from busi_equipment_relationship b, busi_site_tree t "
        "where t.status = '1' and b.equipment_id = t.id "
        "ORDER BY t.site_id"); //,b.transformer_id
    MYSQL_RES *res = pdbHandle->SelectRecord(szSql);
    if (!res)  return;

    string strKey = "";
    MYSQL_ROW row = NULL; 
    multimap<string, ST_DEV_RELATION>::iterator m_pIter;
    while (row = pdbHandle->GetRecord(res))
    {
        ST_DEV_RELATION stRelation;
        memset(&stRelation, 0x00, sizeof(ST_DEV_RELATION));
        if (row[0] && strlen(row[0]) > 0)
        {
            strncpy((char*)stRelation.site_id, (const char*)row[0], sizeof(stRelation.site_id));
        }
        if (row[1] && strlen(row[1]) > 0)
        {
            strncpy((char*)stRelation.m_eqm_code, (const char*)row[1], sizeof(stRelation.m_eqm_code));
        }
        if (row[2] && strlen(row[2]) > 0) {
            strncpy((char*)stRelation.s_eqm_code, (const char*)row[2], sizeof(stRelation.s_eqm_code));
            strKey = row[2];
        }
        if (row[3] && strlen(row[3]) > 0) {
            stRelation.s_eqm_type = atoi((const char*)row[3]);
        }
        if (row[4] && strlen(row[4]) > 0) {
            strncpy((char*)stRelation.s_sys_code, (const char*)row[4], sizeof(stRelation.s_sys_code));            
        }
        stRelation.bClose = 1;
        stRelation.bfault = 0;
        stRelation.wstate = 3;
        mutex_lock(g_map_relation_mutex);
        bool isAdd = false;
        m_pIter = g_map_relation.find(strKey);
        if (m_pIter != g_map_relation.end())
        {
            for (int k = 0; k != (int)g_map_relation.count(strKey); k++, m_pIter++)
            {
                if ((0 == stringncasecmp((const char*)stRelation.site_id,(const char*)((m_pIter->second).site_id),strlen((const char*)(m_pIter->second).site_id))) &&
                    (0 == stringncasecmp((const char*)stRelation.m_eqm_code, (const char*)((m_pIter->second).m_eqm_code), strlen((const char*)(m_pIter->second).m_eqm_code))))
                {
                    isAdd = true;
                    break;
                }
            }
            if (!isAdd) {
                g_map_relation.insert(multimap<string, ST_DEV_RELATION>::value_type(strKey, stRelation));
            }
        }
        else g_map_relation.insert(multimap<string, ST_DEV_RELATION>::value_type(strKey, stRelation));
        mutex_unlock(g_map_relation_mutex);
    }
    pdbHandle->FreeRecord(res);
    return;
}

// 打印设备关系表
static void vPrtDeviceRelationsTable()
{
    int i = 0;
    multimap<string, ST_DEV_RELATION>::iterator m_pIter;
    mutex_lock(g_map_relation_mutex);
    vPrtLogMsg(LOG_DEBUG, RET_OK, "---------Print device Relations Table Config ----------");
    vPrtLogMsg(LOG_DEBUG, RET_OK, " index m_eqm_code\t\t\ts_eqm_code\t\t\t  sType  wstate\tfault\tclose\t");
    m_pIter = g_map_relation.begin();
    while (m_pIter != g_map_relation.end())
    {
        vPrtLogMsg(LOG_DEBUG, RET_OK, "--> %d  %s %s  %d \t%d\t%s\t%s", i + 1,
           // (m_pIter->second).site_id,
            (m_pIter->second).m_eqm_code,
            (m_pIter->second).s_sys_code,
            (m_pIter->second).s_eqm_type,
            (m_pIter->second).wstate,
            ((m_pIter->second).bfault == 0 ? "Normal" : "Fault"),
            ((m_pIter->second).bClose == 0 ? "Opend" : "Close"));  //0:分 1 : 合
        m_pIter++; i++;
    }
    vPrtLogMsg(LOG_DEBUG, RET_OK, "---------- Point Print device Relations_size:%d-----------", g_map_relation.size());
    mutex_unlock(g_map_relation_mutex);
    return;
    //方式1
    /*int k;
    multimap<string, int>::iterator m;
    m = m_map.find(s);
    for (k = 0; k != m_map.count(s); k++, m++)
        cout << m->first << "--" << m->second << endl;
*/
}

// 根据s_eqm_code获取对应的主变设备编码
bool bGetMasterEqmCodeBySubEqmCode(unsigned char *site_id, unsigned char *s_eqm_code, unsigned char *m_eqm_code)
{
    bool isFind = false;
    multimap<string, ST_DEV_RELATION>::iterator m_pIter;
    mutex_lock(g_map_relation_mutex);
    m_pIter = g_map_relation.find((char*)s_eqm_code);
    if (m_pIter != g_map_relation.end())
    {
        for (int c = 0; c < (int)g_map_relation.count((const char*)s_eqm_code); c++, m_pIter++)
        {
            if (0 != stringncasecmp(
                (const char*)site_id,
                (const char*)((m_pIter->second).site_id),
                strlen((const char*)(m_pIter->second).site_id))) continue;

            strcpy((char*)m_eqm_code, (const char*)((m_pIter->second).m_eqm_code));
            isFind = true;
            break;
        }
    }
    mutex_unlock(g_map_relation_mutex);
    return isFind;
}

// 设置运行状态
void flushDevWorkState(unsigned char *site_id, unsigned char *m_eqm_code, unsigned char state)
{
    map<string, ST_DEV_RELATION>::iterator m_pIter;
    mutex_lock(g_map_relation_mutex);
    for (m_pIter = g_map_relation.begin(); m_pIter != g_map_relation.end(); m_pIter++)
    {
        if (0 != stringncasecmp(
            (const char*)site_id,
            (const char*)((m_pIter->second).site_id),
            strlen((const char*)(m_pIter->second).site_id))) continue;
        if (0 != stringncasecmp(
            (const char*)m_eqm_code,
            (const char*)((m_pIter->second).m_eqm_code),
            strlen((const char*)(m_pIter->second).m_eqm_code))) continue;
        (m_pIter->second).wstate = state;
    }
    mutex_unlock(g_map_relation_mutex);
    return ;
}

//设备机组事故状态 state=false:正常  true:事故
bool flushFaultState(unsigned char *site_id, unsigned char *m_eqm_code, unsigned char state)
{
    map<string, ST_DEV_RELATION>::iterator m_pIter;
    mutex_lock(g_map_relation_mutex);
    for (m_pIter = g_map_relation.begin(); m_pIter != g_map_relation.end(); m_pIter++)
    {
        if (0 != stringncasecmp(
            (const char*)site_id,
            (const char*)((m_pIter->second).site_id),
            strlen((const char*)(m_pIter->second).site_id))) continue;
        if (0 != stringncasecmp(
            (const char*)m_eqm_code,
            (const char*)((m_pIter->second).m_eqm_code),
            strlen((const char*)(m_pIter->second).m_eqm_code))) continue;
        (m_pIter->second).bfault = state;
    }
    mutex_unlock(g_map_relation_mutex);
    return true;
}
// 刷新GIS的开关状态
bool flushDevCloseState(unsigned char *site_id, unsigned char *s_eqm_code, unsigned char state)
{
    map<string, ST_DEV_RELATION>::iterator m_pIter;
    mutex_lock(g_map_relation_mutex);
    m_pIter = g_map_relation.find((char*)s_eqm_code);
    if (m_pIter != g_map_relation.end())
    {
        for (int c = 0; c < (int)g_map_relation.count((char*)s_eqm_code); c++, m_pIter++)
        {
            if (0 != stringncasecmp(
                (const char*)site_id,
                (const char*)((m_pIter->second).site_id),
                strlen((const char*)(m_pIter->second).site_id))) continue;
            (m_pIter->second).bClose = state;
        }
    }
    mutex_unlock(g_map_relation_mutex);
    return true;
}
// 获取GIS的开关状态
unsigned char getGisOpenCloseState(unsigned char *site_id, unsigned char *s_eqm_code)
{
    unsigned char isClose = 1;
    map<string, ST_DEV_RELATION>::iterator m_pIter;
    mutex_lock(g_map_relation_mutex);
    m_pIter = g_map_relation.find((char*)s_eqm_code);
    if (m_pIter != g_map_relation.end())
    {
        for (int c = 0; c < (int)g_map_relation.count((char*)s_eqm_code); c++, m_pIter++)
        {
            if (0 != stringncasecmp(
                (const char*)site_id,
                (const char*)((m_pIter->second).site_id),
                strlen((const char*)(m_pIter->second).site_id))) continue;
            isClose = (m_pIter->second).bClose ;
        }
    }
    mutex_unlock(g_map_relation_mutex);
    return isClose;
}

// 从主设备关联关系中，获取pszEqm_code设备的运行状态
unsigned char cGetCurrentWorkState(unsigned char *pszEqm_code)
{
    unsigned char wstate[3] = { 0 };
    map<string, ST_DEV_RELATION>::iterator m_pIter;
    mutex_lock(g_map_relation_mutex);
    m_pIter = g_map_relation.find((char*)pszEqm_code);
    if (m_pIter != g_map_relation.end())
    {   // pszEqm_code 为gis/ce/blq ID时
        for (int c = 0; c < (int)g_map_relation.count((char*)pszEqm_code); c++, m_pIter++)
        {
            //if (0 != stringncasecmp(
            //    (const char*)pszSite_id,
            //    (const char*)((m_pIter->second).site_id),
            //    strlen((const char*)(m_pIter->second).site_id))) continue;
            if (((m_pIter->second).wstate & 0xff) == 1){
                wstate[0] = 1; 
                break;
            }
            else if(((m_pIter->second).wstate & 0xff) == 2){
                wstate[1] = 2;
            }
            else wstate[2] = 3;
        }
    }
    else {
        // 当以主键(从属设备ID)没有找到时，以主变设备ID再查找。
        for (m_pIter = g_map_relation.begin(); m_pIter != g_map_relation.end(); m_pIter++)
        {
            //if (0 != stringncasecmp(
            //    (const char*)pszSite_id,
            //    (const char*)((m_pIter->second).site_id),
            //    strlen((const char*)(m_pIter->second).site_id))) continue;
            if (0 != stringncasecmp(
                (const char*)pszEqm_code,
                (const char*)((m_pIter->second).m_eqm_code),
                strlen((const char*)(m_pIter->second).m_eqm_code))) continue;
            if (((m_pIter->second).wstate & 0xff) == 1){
                wstate[0] = 1;
                break;
            }
            else if (((m_pIter->second).wstate & 0xff) == 2){
                wstate[1] = 2;
            }
            else wstate[2] = 3;
        }
    }
    mutex_unlock(g_map_relation_mutex);
    return (wstate[0] == 1 ? wstate[0] : (wstate[1] == 2 ? wstate[1] : (wstate[2] == 3 ? wstate[2] : 3)));
}
/***************************************************************************
** function name :  thread_cache_proc
** deacription   :  data cache thread
** parameter     :  none
** return code   :  NULL
***************************************************************************/
void *thread_cache_proc(void *arg)   // 同步内存数据线程
{
    while (g_Running)
    {
        vLoadByqDeviceStateTable();
        vLoadBYQThresholdTable(false);
        vLoadDeviceRelationsTable();
        vLoadIECPointTable();
		vLoadIECPointTableNew();
        vLoadImageThreshold();
        //vPrtGisCacheRelation();
        //vPrtBlqCacheRelation();
        vPrtByqDeviceStateTable();
        vPrtBYQThresholdTable();
        vPrtDeviceRelationsTable();
        vPrtIECPointTable();
        vPrtTimeStat();
        vPrtImageThreshold();
        _SLEEP(1000 * 30);
    }
    return NULL;
}

