/****************************************************************************
** File name    :  HTOpencvImg.cpp
** Description  :  define Opencv function
** Create date  :  2018.09.01
** Auther by    :  Liuyx
** Version info :  V1.0.01
** Copyright (C) 2002-2018 xi'an huatek, Inc Co., Ltd
** Update record:
** DATE         AUTHER   DESC
** -------------------------------------------------------------------------
** 2018.09.01   Liuyx    first build
****************************************************************************/
#include "HTOpencvImg.h"

static const char *_FILE_ = "HTOpencvImg.cpp";
static FILE *fpIni = NULL;  // opencv config file handle.
static map<string, ST_OPENCV_CONF>  g_conf_img;

/*************************************************************************
Function iniGetString
Rectives a character string from the specified section in the
specified inifile
Param In     :
pszSection :  address of section
pszEntry   :  address of entry
uiBufLen   :  size of destination buffer
pszFileName:  address of inifile name
Param Out    :
pszRetBuf  :  destination buffer
Return Code  :
0          :  success
<0         :  failed
*************************************************************************/

static int iGetString(const char *pszSection, const char *pszEntry, char *pszRetBuf, unsigned int uiBufLen)
{
    //FILE *fpIni;
    char szBuf[DEF_BUFFER_1K + 1], *psz1, *psz2, *psz;
    int iSectFlag, iLen;

    if (fpIni == NULL) {
        if ((fpIni = fopen(HT_CONFIG_FILE, "r")) == NULL)
            return (-1);
    }
    fseek(fpIni, 0, SEEK_SET);
    /*** check section ***/
    iSectFlag = 0;
    while (!feof(fpIni)) {
        if (fgets(szBuf, DEF_BUFFER_1K, fpIni) == NULL)   break;
        psz = szBuf;

        while (*psz != '[' && *psz != '#' && *psz != '\0')   psz++;
        if (*psz != '[')   continue;
        psz++;
        while (*psz == ' ' || *psz == '\t')   psz++;
        psz1 = psz;
        while (*psz != ']' && *psz != '\0')   psz++;
        if (*psz == '\0')  continue;
        while (*(psz - 1) == ' ' || *(psz - 1) == '\t')   psz--;
        *psz = '\0';
        if (!strcmp(psz1, pszSection)) {
            iSectFlag = 1;
            break;
        }
    }/*** while ***/
    if (!iSectFlag) {
        //fclose(fpIni);
        return (-1);
    }

    /*** check entry ***/
    while (!feof(fpIni))   {
        if (fgets(szBuf, DEF_BUFFER_1K, fpIni) == NULL)   break;
        psz = szBuf;
        while (*psz == ' ' || *psz == '\t')   psz++;
        if (*psz == '#' || *psz == '\0')   continue;
        if (*psz == '[')   break;

        psz1 = psz;
        while (*psz != '=' && *psz != '\0')   psz++;
        if (*psz == '\0')   continue;
        psz2 = psz + 1;
        if (psz1 == psz)   continue;
        while (*(psz - 1) == ' ' || *(psz - 1) == '\t')   psz--;
        *psz = '\0';

#ifdef _WIN32
        if (strcmp(psz1, pszEntry))   continue;
#else
        if (strcasecmp(psz1, pszEntry)) continue;
#endif
        //fclose(fpIni);

        psz = psz2;
        while (*psz == ' ' || *psz == '\t')  psz++;
        psz2 = psz;
        while (*psz != '#' && *psz != '\0' && !(*psz == '/' && (*(psz + 1) == '*' || *(psz + 1) == '/'))) psz++;
        while (*(psz - 1) == ' ' || *(psz - 1) == '\t' || *(psz - 1) == 0x0a || *(psz - 1) == 0x0d)
        {
            *(psz - 1) = '\0';
            psz--;
        }
        //*psz= '\0';
        iLen = strlen(psz2);
        if (psz2[iLen - 1] == 0x0a || psz2[iLen - 1] == 0x0d)  psz2[iLen - 1] = 0x00;
        if (iLen == 0) return (-1);
        if (iLen > (int)uiBufLen)   iLen = uiBufLen;
        memcpy(pszRetBuf, psz2, iLen);
        *(pszRetBuf + iLen) = '\0';
        return (0);
    }
    //fclose(fpIni);
    return (-1);
}

// 加载图片配置文件数据
bool iGetOpencvConf()
{
    int iRet = -1, icount = 0, img_size = 0;
    char szTmp[512], szSection[128], szSite_id[DEF_EQM_CODE_SIZE] = { 0 };
    string strKey;

    // 打开配置文件,准备加载配置参数
    if (fpIni == NULL) {
        if ((fpIni = fopen(HT_CONFIMG_FILE, "r")) == NULL)   {
            vPrtLogMsg(LOG_ERROR, errno, "open config file: %s failed.", HT_CONFIMG_FILE);
            return NULL;
        }
    }

    // 获取站点ID
    memset(szTmp, 0x00, sizeof(szTmp));
    if ((iRet = iGetString("SITE_CODE", "site_id", szTmp, sizeof(szTmp))) < 0) {
        vPrtLogMsg(LOG_ERROR, iRet, "get site_id parameter failed");
        fclose(fpIni);
        return false;
    }
    strcpy((char*)szSite_id, szTmp);

    // 获取站点ID
    memset(szTmp, 0x00, sizeof(szTmp));
    if ((iRet = iGetString("COMMON_CONF", "img_file_min_size", szTmp, sizeof(szTmp))) < 0) {
        vPrtLogMsg(LOG_ERROR, iRet, "get img_file_min_size parameter failed");
        fclose(fpIni);
        return false;
    }
    img_size = atoi(szTmp);

    // 获取配置项的个数
    memset(szTmp, 0x00, sizeof(szTmp));
    if ((iRet = iGetString("CAMERA_COUNT", "CameraCount", szTmp, sizeof(szTmp))) < 0) {
        vPrtLogMsg(LOG_ERROR, iRet, "get CameraCount parameter failed");
        fclose(fpIni);
        return false;
    }
    icount = atoi(szTmp);
    vPrtLogMsg(LOG_DEBUG, 0, "get opencv config count: %d", icount);

    for (int i = 0; i < icount; i++)
    {
        ST_OPENCV_CONF  stConf;
        memset(&stConf, 0x00, sizeof(ST_OPENCV_CONF));
        strcpy((char*)stConf.site_id, szSite_id);   // 站点id
        stConf.img_min_size = img_size;

        snprintf(szSection, sizeof(szSection),"CAMERA_%03d", i + 1);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "eqm_type", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: eqm_type parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.eqm_type[0] = (szTmp[0] & 0xff);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "eqm_code", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: eqm_code parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        strcpy((char*)stConf.eqm_code, szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "system_code", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: system_code parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        strcpy((char*)stConf.sys_code, szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "table_type", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: table_type parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        strcpy((char*)stConf.table_type, szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "name", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: name parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        strcpy((char*)stConf.name, szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "sersorid", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: sersorid parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        strcpy((char*)stConf.sersorid, szTmp);
        strKey = szTmp;

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "base_angle", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: base_angle parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.base_angle = (unsigned int)atoi(szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "meter_type", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: meter_type parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.meter_type = (unsigned char)atoi(szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "meter_units", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: meter_units parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.meter_units = (unsigned char)atoi(szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "direct_Clockwise", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: direct_Clockwise parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.direct_Clockwise = (unsigned char)atoi(szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "init_value", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: init_value parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.dInitVal = (double)atof(szTmp);
        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "max_pointer_len", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: max_pointer_len parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.max_pointer_len = (unsigned int)atoi(szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "img_path", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: img_path parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        strcpy((char*)stConf.img_path, szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "img_bak", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: img_bak parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        strcpy((char*)stConf.img_bak, szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "is_replace", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get is_replace parameter failed");
            return false;
        }
        if (stringcasecmp(szTmp, "true") == 0)
            stConf.is_replace = true;    // 替换原文件
        else stConf.is_replace = false;

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "s_height", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: s_height parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.s_height = atoi(szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "e_height", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: e_height parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.e_height = atoi(szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "s_width", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: s_width parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.s_width = atoi(szTmp);

        memset(szTmp, 0x00, sizeof(szTmp));
        if ((iRet = iGetString(szSection, "e_width", szTmp, sizeof(szTmp))) < 0) {
            vPrtLogMsg(LOG_ERROR, iRet, "get %s: e_width parameter failed", szSection);
            fclose(fpIni);
            return false;
        }
        stConf.e_width = atoi(szTmp);
        stConf.seqno = i + 1;
        g_conf_img.insert(map<string, struct _CONF_IMG>::value_type(strKey, stConf));
    }
    fclose(fpIni);
    return true;
}
// 从文件名截取sersorid
static bool bGetSersorIDOfFilename(char *filename, char *sersorid, char *pExt)
{
    char *p = NULL, *s = NULL, *t = NULL;
    // 文件名格式：sersorID_time.jpg 或 ../img/sersorid_time.jpg
    if (filename && strlen(filename) >= 17) {
        p = filename + strlen((const char*)filename);
        while (*p != '\\' && *p != '/'  && p != filename) {
            if (*p == '.')  t = p + 1;  // 扩展名位置
            if (*p == '_')  s = p;  // 分隔符_位置
            p--;
        }
        if (t) {
            strcpy(pExt, t);
            pExt[strlen(pExt)] = 0x00;
        }
        if (s) {
            memcpy(sersorid, p + 1, s - p - 1);
            sersorid[s - p - 1] = 0x00;
        }
        return true;
    }
    return false;
}
// 移动文件到指定目录（备份文件）
bool bBakFilename(char *filename, char *pBakPath)
{
    char *p = NULL, *s = NULL, *t = NULL;
    char szCmd[1024] = { 0 };
    // 文件名格式：sersorID_time.jpg 或 ../img/sersorid_time.jpg
    if (filename && pBakPath)
    {
        if (iDirOrFileExist(pBakPath))
            iBuildDirent(pBakPath);
#ifdef _WIN32
        snprintf(szCmd, sizeof(szCmd), "move %s %s", filename, pBakPath);
#else
        snprintf(szCmd, sizeof(szCmd),"mv %s %s", filename, pBakPath);
#endif
        system(szCmd);
        return true;
    }
    return false;
}
// 显示图片
void showImg(const char *title, Mat &img)
{
#ifdef _IM_SHOW
    imshow(title, img);
    waitKey(0);
#endif
}

int dcmp(double x)
{
    if (fabs(x) < eps) return 0;
    else return x < 0 ? -1 : 1;
}

//向量点积
double Dot(Point A, Point B)
{
    return A.x*B.x + A.y*B.y;
}

//向量模长
double Length(Point A)
{
    return sqrt(Dot(A, A));
}

//向量叉积
double Cross(Point A, Point B)
{
    return A.x*B.y - A.y*B.x;
}
//点到线段的距离
double DistancetoSegment(Point P, Point A, Point B)
{
    if (A == B)return Length(P - A);
    Point v1 = B - A, v2 = P - A, v3 = P - B;
    if (dcmp(Dot(v1, v2))<0)  return  Length(v2);
    else if (dcmp(Dot(v1, v3))>0)  return Length(v3);
    else return (fabs(Cross(v1, v2)) / Length(v1));
}


/* 获取指定目录下的*.xxx文件
pPath   :   path, eg: /usr/local/img/*.jpg
: "D:\\Photo and Video\\destop1\\*.jpg";
*/
int iGetImgFileList(const char *pPath, std::vector<cv::String> & image_files)
{
    cv::String pattern_jpg = (char*)pPath;

    //std::vector<cv::String> image_files;
    try{
        cv::glob(pattern_jpg, image_files);
        if (image_files.size() == 0) {
            //std::cout << "No image files[jpg]" << std::endl;
            return 0;
        }
        return (int)image_files.size();
    }
    catch (cv::Exception &e)
    {
        printf("Get ImgFile List exception,msg:%s\n", e.err.c_str());
    }
    return 0;
}

//下面是以海康威视的某款网络摄像头为例，使用 OpenCV 的 VideoCapture 类来显示实时视频
// rtsp://[username]:[password]@[ip]:[port]/[codec]/[channel]/[subtype]/av_stream
// 1) username  用户名，常用 admin
// 2) password  密码，常用 12345
// 3) ip        摄像头IP，如 192.0.0.64
// 4) port      端口号，默认为 554
// 5) codec     视频编码模式，有 h264、MPEG - 4、mpeg4 等
// 6) channel   通道号，起始为1，例如通道1，则为 ch1
// 7) subtype   码流类型，主码流为 main，辅码流为 sub
int showRtspVideoStream(const char* rtsp_url)
{
    // "rtsp://admin:a1234567@192.168.5.186:554/MPEG-4/ch1/main/av_stream";
    String rtsp_addr = rtsp_url;

    VideoCapture cap(rtsp_addr);
    if (!cap.isOpened()) return -1;

    // cap.open(rtsp_addr);
    Mat frame;
    for (;;) {
        cap >> frame;
        if (frame.empty())   break;
        imshow("Video Stream", frame);
        if (waitKey(10) == 'q')  break;
    }
    cap.release(); // 关闭视频流。
    return 0;
}

// 播放视频流
int showRtspVideoStreams(const char *rtsp_url)
{
    //打开视频文件：其实就是建立一个VideoCapture结构
    VideoCapture capture(rtsp_url);
    //检测是否正常打开:成功打开时，isOpened返回ture
    if (!capture.isOpened()) {
        cout << "fail to open!" << endl;
        return 0;
    }

    //获取整个帧数
    long totalFrameNumber = (long)capture.get(CV_CAP_PROP_FRAME_COUNT);
    cout << "整个视频共" << totalFrameNumber << "帧" << endl;

    //设置开始帧()
    long frameToStart = 300;
    //capture.set( CV_CAP_PROP_POS_FRAMES, frameToStart);
    cout << "从第" << frameToStart << "帧开始读" << endl;

    //设置结束帧
    int frameToStop = 400;

    if (frameToStop < frameToStart)
    {
        cout << "结束帧小于开始帧，程序错误，即将退出！" << endl;
        return -1;
    }
    else
    {
        cout << "结束帧为：第" << frameToStop << "帧" << endl;
    }

    //获取帧率
    double rate = capture.get(CV_CAP_PROP_FPS);
    cout << "帧率为:" << rate << endl;

    //定义一个用来控制读取视频循环结束的变量
    bool stop = false;
    //承载每一帧的图像
    Mat frame;
    //显示每一帧的窗口
    namedWindow("Extracted frame");
    //两帧间的间隔时间:
    int delay = 1000 / (int)rate;

    //利用while循环读取帧
    //currentFrame是在循环体中控制读取到指定的帧后循环结束的变量
    long currentFrame = frameToStart;

    //滤波器的核
    int kernel_size = 3;
    Mat kernel = Mat::ones(kernel_size, kernel_size, CV_32F) / (float)(kernel_size*kernel_size);

    while (!stop)
    {
        //读取下一帧
        if (!capture.read(frame))
        {
            cout << "读取视频失败" << endl;
            return -1;
        }
        //这里加滤波程序
        imshow("Extracted frame", frame);
        //filter2D(frame, frame, -1, kernel);

        //imshow("after filter", frame);
        //cout << "正在读取第" << currentFrame << "帧" << endl;
        //waitKey(int delay=0)当delay ≤ 0时会永远等待；当delay>0时会等待delay毫秒
        //当时间结束前没有按键按下时，返回值为-1；否则返回按键

        int c = waitKey(delay);
        //按下ESC或者到达指定的结束帧后退出读取视频
        if ((char)c == 27 /*|| currentFrame > frameToStop*/)
        {
            stop = true;
        }
        //按下按键后会停留在当前帧，等待下一次按键
        if (c >= 0)
        {
            waitKey(0);
        }
        currentFrame++;
    }
    //关闭视频文件
    capture.release();
    return 0;
}
// 截取图片指定区域并保存
// isReplace = true:  覆盖原文件,不保留原文件,文件名不变
//           = false: 保留原文件,并将截图文件命名为 xxx_cut.xxx
int vCutImageSave(ST_OPENCV_CONF *stConf, char* path_filename)
{
    char szDest[MAX_PATH] = { 0 }, ext[8] = { 0 };
    int shigh, ehigh, swidth, ewidth;
    Mat image;
    try{
        image = imread(path_filename);
    }
    catch (cv::Exception &e)
    {
        printf("imread exception,msg:%s\n", e.err.c_str());
        return RET_FAIL;
    }
    int w = image.cols; // 列数
    int h = image.rows; // 行数
    vPrtLogMsg(LOG_DEBUG, RET_OK, "file:%s, source high=%d  width=%d", path_filename, h, w);

    shigh = stConf->s_height;
    ehigh = stConf->e_height;
    swidth = stConf->s_width;
    ewidth = stConf->e_width;
    if (shigh > h) shigh = 10;       // 起始高度>总高度时,起始高度=10
    if (ehigh > h) ehigh = h;        // 结束高度>总高度时,结束高度=h
    if (swidth > w) swidth = 10;     // 结束宽度>总宽度时,起始宽度=10
    if (ewidth > w) ewidth = w;      // 结束宽度>总宽度时,结束宽度=w
    //showImg("SRC:", image);
    //        0列-h-y             27列-w-x
    //   0行  +-------------------------+
    //   1行  |                         |
    //   2行  |                         |
    //   3行  |                         |
    //   4行  |                         |
    //   5行  +-------------------------+ 5行,27列
    //
    //      Range(起始行，结束行), Range(起始列，结束列),
    Mat image_roi(image, Range(shigh, ehigh), Range(swidth, ewidth));
    if (image_roi.empty()) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "cut picture:%s to high=(%d,%d) width=(%d,%d) failed.", path_filename, shigh, ehigh, swidth, ewidth);
        return RET_FAIL;
    }
    showImg("CUT PIC:", image_roi);
    if (!stConf->is_replace)
    {
        bGetSersorIDOfFilename(path_filename, szDest, ext);
        // 保留原文件,并重命名
        snprintf(szDest, sizeof(szDest), "%s", path_filename);
        memset(szDest + strlen(szDest) - strlen(ext) - 1, 0x00, sizeof(szDest)-strlen(szDest) + 1);
        strcat(szDest, "_cut");
        snprintf(szDest + strlen(szDest), sizeof(szDest), ".%s", ext);
        imwrite(szDest, image_roi);
        strcpy(path_filename, szDest);  // 返回新文件名
    }
    else imwrite(path_filename, image_roi);  // 不保留原文件，覆盖原文件

    return RET_OK;
}

// 查表获取指针读数
static double dGetReadValueOf3ex5050(double du, double base, int linelen)
{
    int i = 0, min_id = -1;
    double tmp_du = du, min_du = 30;
    double val_inter = 0;

    if (du > base)  {
        tmp_du = ((int)(du * 100) % (int)(base * 100)) / 100;// +(du / base);
    }

    while (g_YB_3EX5050[i].du > 0)
    {
        // 找du数差值最小的，也就是最接近的记录。
        if (tmp_du == g_YB_3EX5050[i].du) return g_YB_3EX5050[i].val;
        if (tmp_du == g_YB_3EX5050[i + 1].du) return g_YB_3EX5050[i + 1].val;

        if (fabs(tmp_du - g_YB_3EX5050[i].du) < min_du) {
            min_du = fabs(tmp_du - g_YB_3EX5050[i].du);
            min_id = i;
        }
        i++;
    }
    if (min_id < 0) return 0.0f;  // 没找到
    if (g_YB_3EX5050[min_id + 1].val > 0.0)
        val_inter = fabs(g_YB_3EX5050[min_id + 1].val - g_YB_3EX5050[min_id].val) / 10;
    else
        val_inter = fabs(g_YB_3EX5050[min_id].val - g_YB_3EX5050[min_id - 1].val) / 10;
    if (min_du > 10.0)
        return g_YB_3EX5050[min_id].val + (val_inter * min_du);
    return fabs(g_YB_3EX5050[min_id].val - (val_inter * min_du));
}

// 查表获取指针读数
static double dGetReadValueOfJSY10(double du, double base, int linelen)
{
    int i = 0, min_id = -1;
    double tmp_du = du, min_du = 30;
    double val_inter = 0;

    if (du > base)  {
        tmp_du = ((int)(du * 100) % (int)(base * 100)) / 100;// +(du / base);
    }

    while (g_YB_JSY10[i].du > 0)
    {
        // 找du数差值最小的，也就是最接近的记录。
        if (tmp_du == g_YB_JSY10[i].du) return g_YB_JSY10[i].val;
        if (tmp_du == g_YB_JSY10[i + 1].du) return g_YB_JSY10[i + 1].val;

        if (fabs(tmp_du - g_YB_JSY10[i].du) < min_du) {
            min_du = fabs(tmp_du - g_YB_JSY10[i].du);
            min_id = i;
        }
        i++;
    }
    if (min_id < 0) return 0.0f;  // 没找到
    if (g_YB_JSY10[min_id + 1].val > 0.0)
        val_inter = fabs(g_YB_JSY10[min_id + 1].val - g_YB_JSY10[min_id].val) / 10;
    else
        val_inter = fabs(g_YB_JSY10[min_id].val - g_YB_JSY10[min_id - 1].val) / 10;
    return g_YB_JSY10[min_id].val;//  +val_inter;
}
// 查表获取油表指针读数
double dGetReadValueOfOCR17NI12MO2_16(double du, double base, int linelen)
{
    int i = 0, min_id = -1;
    double tmp_du = du, min_du = 3.3f;  // ((360-90)/10)/2=13.5，单个区域差13.5°
    double val_inter = 0;

    if (du > base)  {
        tmp_du = ((int)(du * 100) % (int)(base * 100)) / 100;// +(du / base);
    }
    while (g_YB_OCR17NI12MO2_16[i].du > 0)
    {
        // 找du数差值最小的，也就是最接近的记录。
        if (tmp_du == g_YB_OCR17NI12MO2_16[i].du) return g_YB_OCR17NI12MO2_16[i].val;
        if (tmp_du == g_YB_OCR17NI12MO2_16[i + 1].du) return g_YB_OCR17NI12MO2_16[i + 1].val;

        if (fabs(tmp_du - g_YB_OCR17NI12MO2_16[i].du) < min_du) {
            min_du = fabs(tmp_du - g_YB_OCR17NI12MO2_16[i].du);
            min_id = i;
        }
        i++;
    }
    if (min_id < 0) return -1.0f;  // 没找到
    //printf("Base = %.2f°-%.2f   -- input:%.2f\n", g_YB_YZFTH[min_id].du, g_YB_YZFTH[min_id].val, tmp_du);
    if (tmp_du > g_YB_OCR17NI12MO2_16[min_id].du) {   // 当前度大于最近的度时
        val_inter = g_YB_OCR17NI12MO2_16[min_id].val + 0.02; // (min_du * 0.02 / 10);  // 可能会出现负数
        //printf("(%.3f-%.3f)    min_du=%.3f\n", g_YB_YZFTH[min_id].val, (min_du * 0.1 / 10), min_du);
    }
    else {
        val_inter = g_YB_OCR17NI12MO2_16[min_id].val + (min_du * 0.02 / 10);
        //printf("(%.3f+%.3f)    min_du=%.3f\n", g_YB_YZFTH[min_id].val, (min_du * 0.1 / 10), min_du);
    }
    return val_inter;
}
// 查表获取油表指针读数
double dGetReadValueOfOCR17NI12MO2_25(double du, double base, int linelen)
{
    int i = 0, min_id = -1;
    double tmp_du = du, min_du = 3.3f;  // ((360-90)/10)/2=13.5，单个区域差13.5°
    double val_inter = 0;

    if (du > base)  {
        tmp_du = ((int)(du * 100) % (int)(base * 100)) / 100;// +(du / base);
    }
    while (g_YB_OCR17NI12MO2_25[i].du > 0)
    {
        // 找du数差值最小的，也就是最接近的记录。
        if (tmp_du == g_YB_OCR17NI12MO2_25[i].du) return g_YB_OCR17NI12MO2_25[i].val;
        if (tmp_du == g_YB_OCR17NI12MO2_25[i + 1].du) return g_YB_OCR17NI12MO2_25[i + 1].val;

        if (fabs(tmp_du - g_YB_OCR17NI12MO2_25[i].du) < min_du) {
            min_du = fabs(tmp_du - g_YB_OCR17NI12MO2_25[i].du);
            min_id = i;
        }
        i++;
    }
    if (min_id < 0) return -1.0f;  // 没找到
    //printf("Base = %.2f°-%.2f   -- input:%.2f\n", g_YB_YZFTH[min_id].du, g_YB_YZFTH[min_id].val, tmp_du);
    if (tmp_du > g_YB_OCR17NI12MO2_25[min_id].du) {   // 当前度大于最近的度时
        val_inter = g_YB_OCR17NI12MO2_25[min_id].val + (min_du * 0.05 / 10);  // 可能会出现负数
        //printf("(%.3f-%.3f)    min_du=%.3f\n", g_YB_YZFTH[min_id].val, (min_du * 0.1 / 10), min_du);
    }
    else {
        val_inter = g_YB_OCR17NI12MO2_25[min_id].val + (min_du * 0.05 / 10);
        //printf("(%.3f+%.3f)    min_du=%.3f\n", g_YB_YZFTH[min_id].val, (min_du * 0.1 / 10), min_du);
    }
    return val_inter;
}
// 100摄氏度的油温表读数
double dReadPointValue_Wss411_100(double du, double base, int linelen)
{
    int i = 0, min_id = -1;
    double tmp_du = du, min_du = 3.4f;  // ((360-90)/10)/2=13.5，单个区域差13.5°
    double val_inter = 0;

    if (du > base)  {
        tmp_du = ((int)(du * 100) % (int)(base * 100)) / 100;// +(du / base);
    }
    // if (tmp_du < 45)  return 10.0f;
    // if (tmp_du > 315) return 0.0f;  // 无效范围内的角度值,默认给最大读数.系统提示告警
    if (tmp_du == 0) tmp_du = 180.0f;
    while (g_YB_WSS100[i].du > 0)
    {
        // 找du数差值最小的，也就是最接近的记录。
        if (tmp_du == g_YB_WSS100[i].du) return g_YB_WSS100[i].val;
        if (tmp_du == g_YB_WSS100[i + 1].du) return g_YB_WSS100[i + 1].val;

        if (fabs(tmp_du - g_YB_WSS100[i].du) < min_du) {
            min_du = fabs(tmp_du - g_YB_WSS100[i].du);
            min_id = i;
        }
        i++;
    }
    if (min_id < 0) return -1.0f;  // 没找到
    //printf("Base = %.2f°-%.2f   -- input:%.2f\n", g_YB_YZFTH[min_id].du, g_YB_YZFTH[min_id].val, tmp_du);
    if (tmp_du > g_YB_WSS100[min_id].du) {   // 当前度大于最近的度时
        val_inter = g_YB_WSS100[min_id].val - (min_du * 2 / 10);  // 可能会出现负数
    }
    else {
        val_inter = g_YB_WSS100[min_id].val + (min_du * 2 / 10);
    }
    return val_inter;
}
// 150摄氏度的油温表读数
double dGetReadValueOf_WSS150(double du, double base, int linelen)
{
    int i = 0, min_id = -1;
    double tmp_du = du, min_du = 3.4f;  // ((360-90)/10)/2=13.5，单个区域差13.5°
    double val_inter = 0;

    if (du > base)  {
        tmp_du = ((int)(du * 100) % (int)(base * 100)) / 100;// +(du / base);
    }
   // if (tmp_du < 45)  return 10.0f;
   // if (tmp_du > 315) return 0.0f;  // 无效范围内的角度值,默认给最大读数.系统提示告警

    while (g_YB_WSS150[i].du > 0)
    {
        // 找du数差值最小的，也就是最接近的记录。
        if (tmp_du == g_YB_WSS150[i].du) return g_YB_WSS150[i].val;
        if (tmp_du == g_YB_WSS150[i + 1].du) return g_YB_WSS150[i + 1].val;

        if (fabs(tmp_du - g_YB_WSS150[i].du) < min_du) {
            min_du = fabs(tmp_du - g_YB_WSS150[i].du);
            min_id = i;
        }
        i++;
    }
    if (min_id < 0) return -1.0f;  // 没找到
    //printf("Base = %.2f°-%.2f   -- input:%.2f\n", g_YB_YZFTH[min_id].du, g_YB_YZFTH[min_id].val, tmp_du);
    if (tmp_du > g_YB_WSS150[min_id].du) {   // 当前度大于最近的度时
        val_inter = g_YB_WSS150[min_id].val - (min_du * 3 / 10);  // 可能会出现负数
        //printf("(%.3f-%.3f)    min_du=%.3f\n", g_YB_YZFTH[min_id].val, (min_du * 0.1 / 10), min_du);
    }
    else {
        //val_inter = g_YB_WSS150[min_id].val + (min_du * 3 / 10);
        val_inter = g_YB_WSS150[min_id].val + min_du ;
        //printf("(%.3f+%.3f)    min_du=%.3f\n", g_YB_YZFTH[min_id].val, (min_du * 0.1 / 10), min_du);
    }
    return val_inter;
}
//double gtest(double du, double base, int linelen)
//{
//    return dGetReadValueOfYZFTH( du,  base,  linelen);
//}
// 遍历所有像素点1
void colorReduce(cv::Mat &inputImage, int div)
{
    // 参数准备  
    // outputImage = inputImage.clone();

    int rowNumber = inputImage.rows;
    int colNumber = inputImage.cols*inputImage.channels();
    if (inputImage.isContinuous()) {  // 判断数据在内存是否连续存储的。
        colNumber = colNumber * rowNumber;
        rowNumber = 1;
    }
    for (int i = 0; i < rowNumber; i++)
    {
        // 获取第i行的首地址  
        uchar* data = inputImage.ptr<uchar>(i);

        for (int j = 0; j < colNumber; j++)  // 列循环  
        {
            // 开始处理每一个像素值，每一个像素值都减去div  
            data[j] = data[j] - div;
        }
    }
}
// using .ptr and * ++ and bitwise (continuous+channels)
void colorReduce7(cv::Mat &image, int div = 64) {
    int nr = image.rows; // number of rows
    int nc = image.cols; // number of columns
    if (image.isContinuous())  {
        // then no padded pixels
        nc = nc*nr;
        nr = 1;  // it is now a 1D array
    }
    int n = static_cast<int>(log(static_cast<double>(div)) / log(2.0));
    // mask used to round the pixel value
    uchar mask = 0xFF << n; // e.g. for div=16, mask= 0xF0
    for (int j = 0; j < nr; j++) {
        uchar* data = image.ptr<uchar>(j);
        for (int i = 0; i < nc; i++) {
            if (image.channels() == 1)
                *data++ = *data & mask + div / 2;
            else if (image.channels() == 3) {
                *data++ = *data & mask + div / 2;
                *data++ = *data & mask + div / 2;
                *data++ = *data & mask + div / 2;
            }
        } // end of row                 
    }
}
void colorReduce(cv::Mat & inputImage)
{
    // 参数准备  
    int a, b, c;
    int rowNumber = inputImage.rows;
    int colNumber = inputImage.cols*inputImage.channels();
    if (inputImage.isContinuous()) {  // 判断数据在内存是否连续存储的。
        colNumber = colNumber * rowNumber;
        rowNumber = 1;
    }
    for (int i = 0; i < rowNumber; i++)  {
        for (int j = 0; j < colNumber; j++)
        {
            a = inputImage.at<Vec3b>(i, j)[0];
            b = inputImage.at<Vec3b>(i, j)[1];
            c = inputImage.at<Vec3b>(i, j)[2];
            if (a > 180)  inputImage.at<Vec3b>(i, j)[0] = 0;
            if (b < 50)   inputImage.at<Vec3b>(i, j)[1] = 255;
            if (c < 120)  inputImage.at<Vec3b>(i, j)[2] = 0;
        }
    }
}
// 遍历所有像素点2
Mat inverseColor4(cv::Mat &srcImage)
{
    // 初始化原图像迭代器  
    MatIterator_<Vec3b> srcIterStart = srcImage.begin<Vec3b>();
    MatIterator_<Vec3b> srcIterEnd = srcImage.end<Vec3b>();

    while (srcIterStart != srcIterEnd)
    {
        (*srcIterStart)[0] = 255 - (*srcIterStart)[0];
        (*srcIterStart)[1] = 255 - (*srcIterStart)[1];
        (*srcIterStart)[2] = 255 - (*srcIterStart)[2];
        srcIterStart++;
    }
    return srcImage;
}

// 读取仪表指针数值
double dReadPointValues(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = 0.0;
    Mat temp, dst, gray;                   //src-->temp-->dst(src为原图；temp是src经canny提取边缘的图，
    //用于下面霍夫变换；dst是最终结果图，要在temp灰度图的基础上变为彩色图才能呈现画线效果）
    //rows=行 rols=列

    Mat src = imread(jpg_file, 1);    //读取图片到mat
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    showImg("src:", src);
    colorReduce7(src);
    showImg("cvtColor dst:", dst);
    vPrtLogMsg(LOG_DEBUG, RET_OK, "imread picture file %s size:[%dx%d],channel:%d", jpg_file, src.cols, src.rows, src.channels());

    Canny(src, temp, 20, 80, 3);    //提取边缘(如果不边缘提取就会浪费巨大时间）
    //showImg("Canny temp:", temp);
    cvtColor(temp, dst, CV_GRAY2BGR);//将边缘提取的灰度图转换为BGR图便于画线 CV_GRAY2BGR CV_BGR2GRAY
    colorReduce7(src);
    showImg("cvtColor dst:", dst);
    //储存检测圆的容器  
    vector<Vec3f> circles;

    //调用Hough变换检测圆  
    //参数为：待检测图像，检测结果，检测方法（这个参数唯一）,累加器的分辨率，两个圆间的距离，canny门限的上限（下限自动设为上限的一半），圆心所需要的最小的投票数，最大和最小半径  
    HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, 50, 200, 100, 100, 300);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 5, 150, 350, 50, 200);
    if (circles.size() == 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "picture file :%s can't find circles point. return.", jpg_file);
        return -1.0;
    }
    showImg("HoughCircles TEMP:", temp);
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < temp.rows && f[0] - f[2] >= 0 && f[1] + f[2]<temp.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }

    Point center((int)circles[pos][0], (int)circles[pos][1]);  //找到的圆心
    int   radius = (int)circles[pos][2];                       //找到的半径
    circle(dst, center, radius, Scalar(255, 0, 0), 2);  // BGR=blue
    showImg("HoughCircles dst:", dst);
    double vv = 0;
    list<MyLine> list_MyLine;
    vector<Vec4i> lines2;                           //线段检测
    HoughLinesP(temp, lines2, 1, CV_PI / 180, 50, 50, 10);
    if (lines2.size() == 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s can't find line. return.", jpg_file);
        return -1.0;
    }

    vPrtLogMsg(LOG_DEBUG, RET_OK, "file :%s find Cirles=%d,line=%d", jpg_file, (int)circles.size(), (int)lines2.size());
    for (size_t i = 0; i < lines2.size(); i++)
    {
        Vec4i l = lines2[i];
        Point A(l[0], l[1]), B(l[2], l[3]);
        vv = DistancetoSegment(center, A, B);
        //if (vv < stConf->max_pointer_len)//根据圆心到指针的距离阈值滤掉其他线段
        if (vv < 30)//根据圆心到指针的距离阈值滤掉其他线段
        {
            bool down = (A.y + B.y - 2 * center.y>0);//判断长的在过圆心的水平线上部还是下部
            if (A.x == B.x)  //斜率为无穷的情况
            {
                list_MyLine.push_back(MyLine(i, 90 + (down ? 180 : 0), (int)Length(Point(A.x - B.x, A.y - B.y))));
            }
            else if (A.y == B.y) //水平的情况
            {
                list_MyLine.push_back(MyLine(i, A.x + B.x - 2 * center.x > 0 ? 0 : 180, (int)Length(Point(A.x - B.x, A.y - B.y))));
            }
            else
            {
                if (down){
                    if (A.y > center.y)
                        list_MyLine.push_back(MyLine(i, 360 - (int)(atan2(A.y - B.y, A.x - B.x) * 180 / PI), (int)Length(Point(A.x - B.x, A.y - B.y))));
                    else
                        list_MyLine.push_back(MyLine(i, 360 - (int)(atan2(B.y - A.y, B.x - A.x) * 180 / PI), (int)Length(Point(A.x - B.x, A.y - B.y))));
                }
                else {
                    if (A.y < center.y)
                        list_MyLine.push_back(MyLine(i, abs((int)(atan2(A.y - B.y, A.x - B.x) * 180 / PI)), (int)Length(Point(A.x - B.x, A.y - B.y))));
                    else
                        list_MyLine.push_back(MyLine(i, abs((int)(atan2(B.y - A.y, B.x - A.x) * 180 / PI)), (int)Length(Point(A.x - B.x, A.y - B.y))));
                }
            }
            //line(dst, A, B, Scalar(0, 0, i * 20 + 40), 2, CV_AA);
            line(dst, A, B, Scalar(0, 255, 0), 2, CV_AA);  //gree
        }
    }
    showImg("dst", dst);

    //根据角度区分所属指针
    int now_k, pre_k = 720;//当前线段的角度和前一个线段的角度
    int num = 0;//指针数（可能为2或3）
    int Du[3] = { 0 };//3个指针的度数（每组的平均）
    int Le[3] = { 0 };//Le[i]=Le_ping[i]*0.2+le_max[i]*0.8;
    int Le_ping[3] = { 0 };//3个指针的长度（每组的平均）
    int Le_max[3] = { 0 };//3个指针的长度（每组区最大的）
    int t_num = 0;//每组的数量（求平均用）

    MyLine now_Line;
    list_MyLine.push_back(MyLine(99, 888, 0));//向其中插入一个右边界这样方便处理
    list_MyLine.sort();
    while (!list_MyLine.empty())
    {
        now_Line = list_MyLine.front();
        now_k = now_Line.k;
        //if (abs(now_k - pre_k)>5)//两个角度之差小于5°的算是同一类
        if (abs(now_k - pre_k) > 10)//两个角度之差小于10°的算是同一类
        {
            if (num != 0)  //对本组的度数和长度求平均
            {
                Du[num - 1] /= t_num;
                Le_ping[num - 1] /= t_num;
                Le[num - 1] = (int)(Le_ping[num - 1] * 0.2 + Le_max[num - 1] * 0.8);
            }
            if (now_k == 888)  break;//右边界直接跳出
            t_num = 0;//重新统计下一组
            num++; //组数增加1
            if (num > 3) break;
        }
        t_num++;//组内多一条线
        Du[num - 1] += now_Line.k;
        Le_ping[num - 1] += now_Line.l;
        if (now_Line.l > Le_max[num - 1])  Le_max[num - 1] = now_Line.l;
        //now_Line.print();
        vPrtLogMsg(LOG_DEBUG, RET_OK, "Point id: %d   k: %d°   len: %d", now_Line.id, now_Line.k, now_Line.l);
        //printf("Point id: %d   k: %d°   len: %d\n", now_Line.id, now_Line.k, now_Line.l);
        list_MyLine.pop_front();
        pre_k = now_k;
    }

    int t, duval = 0;
    for (int i = 0; i < num - 1; i++)
    {
        for (int j = i + 1; j < num; j++)
        {
            if (Du[i] < Du[j])
            {
                t = Le[i], Le[i] = Le[j], Le[j] = t;
                t = Du[i], Du[i] = Du[j], Du[j] = t;
            }//if end
        }//for end
    }//for end

    // 根据配置文件的仪表类型，查找指针度数对应的读数

    //printf("\nIn Point id: num:%d  k: %d°   len: %d\n", num, Du[0], Le[0]);

    for (int n = 0; n < num; n++) {
        val = dGetReadValueOf3ex5050(Du[n], (double)stConf->base_angle, Le[n]);
        if (val <= 0 || val > 30) continue;
        break;
    }
    //if (val <= 0 && num && num <= 3 ) {
    //    printf("get val over, to tyr get...du=%d\n", Du[1]);
    //    val = dGetReadValueOf3ex5050(Du[1], (double)stConf->base_angle, Le[1]);
    //}
    //if (val > 30) {
    //    printf("get val over, to tyr get...du=%d\n", Du[2]);
    //    val = dGetReadValueOf3ex5050(Du[2], (double)stConf->base_angle, Le[2]);
    //}
    vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s Point=%d  ===  read number is: %.3f", jpg_file, num, val);
    return val;
}
// 读取仪表指针数值
double dReadPointValue_3EX5050(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = 0.0;
    Mat temp, dst, gray;                   //src-->temp-->dst(src为原图；temp是src经canny提取边缘的图，
    //用于下面霍夫变换；dst是最终结果图，要在temp灰度图的基础上变为彩色图才能呈现画线效果）
    //rows=行 rols=列

    Mat src = imread(jpg_file, 0);    //读取图片到mat
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    vPrtLogMsg(LOG_DEBUG, RET_OK, "imread picture file %s size:[%dx%d],channel:%d", jpg_file, src.cols, src.rows, src.channels());
    Canny(src, temp, 20, 80, 3);    //提取边缘(如果不边缘提取就会浪费巨大时间）
    cvtColor(temp, dst, CV_GRAY2BGR);//将边缘提取的灰度图转换为BGR图便于画线 CV_GRAY2BGR CV_BGR2GRAY
    //储存检测圆的容器  
    vector<Vec3f> circles;

    //调用Hough变换检测圆  
    //参数为：待检测图像，检测结果，检测方法（这个参数唯一）,累加器的分辨率，两个圆间的距离，canny门限的上限（下限自动设为上限的一半），圆心所需要的最小的投票数，最大和最小半径  
  //HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, 50, 200, 100, 100, 300);
    HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, 50, 200, 50, 100, 200);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 5, 150, 350, 50, 200);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 6, 100, 300, 50, 200);
    //if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 7, 80, 240, 50, 200);
    //if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 50, 110, 50, 200);
    //if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 150, 300, 50, 100);
    //if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 100, 200, 40, 80);
    //if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 80, 160, 30, 70);
    //if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 50, 100, 20, 60);
    if (circles.size() == 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "picture file :%s can't find circles point. return.", jpg_file);
        return -1.0;
    }
    showImg("HoughCircles TEMP:", temp);
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < temp.rows && f[0] - f[2] >= 0 && f[1] + f[2]<temp.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }

    Point center((int)circles[pos][0], (int)circles[pos][1]);  //找到的圆心
    int   radius = (int)circles[pos][2];                       //找到的半径
    circle(dst, center, radius, Scalar(255, 0, 0), 2);  // BGR=blue
    showImg("HoughCircles dst:", dst);
    double vv = 0;
    list<MyLine> list_MyLine;
    vector<Vec4i> lines2;                           //线段检测
    HoughLinesP(temp, lines2, 1, CV_PI / 180, 50, 50, 10);
    if (lines2.size() == 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s can't find line. return.", jpg_file);
        return -1.0;
    }

    vPrtLogMsg(LOG_DEBUG, RET_OK, "file :%s find Cirles=%d,line=%d", jpg_file, (int)circles.size(), (int)lines2.size());
    for (size_t i = 0; i < lines2.size(); i++)
    {
        Vec4i l = lines2[i];
        Point A(l[0], l[1]), B(l[2], l[3]);
        vv = DistancetoSegment(center, A, B);
        //if (vv < stConf->max_pointer_len)//根据圆心到指针的距离阈值滤掉其他线段
        if (vv < 80)//根据圆心到指针的距离阈值滤掉其他线段
        {
            bool down = (A.y + B.y - 2 * center.y>0);//判断长的在过圆心的水平线上部还是下部
            if (A.x == B.x)  //斜率为无穷的情况
            {
                //list_MyLine.push_back(MyLine(i, 90 + (down ? 180 : 0), (int)Length(Point(A.x - B.x, A.y - B.y))));
            }
            else if (A.y == B.y) //水平的情况
            {
                //list_MyLine.push_back(MyLine(i, A.x + B.x - 2 * center.x > 0 ? 0 : 180, (int)Length(Point(A.x - B.x, A.y - B.y))));
            }
            else
            {
                if (down){
                    if (A.y > center.y)
                        list_MyLine.push_back(MyLine(i, 360 - (int)(atan2(A.y - B.y, A.x - B.x) * 180 / PI), (int)Length(Point(A.x - B.x, A.y - B.y))));
                    else
                        list_MyLine.push_back(MyLine(i, 360 - (int)(atan2(B.y - A.y, B.x - A.x) * 180 / PI), (int)Length(Point(A.x - B.x, A.y - B.y))));
                }
                else {
                    if (A.y < center.y)
                        list_MyLine.push_back(MyLine(i, abs((int)(atan2(A.y - B.y, A.x - B.x) * 180 / PI)), (int)Length(Point(A.x - B.x, A.y - B.y))));
                    else
                        list_MyLine.push_back(MyLine(i, abs((int)(atan2(B.y - A.y, B.x - A.x) * 180 / PI)), (int)Length(Point(A.x - B.x, A.y - B.y))));
                }
            }
            //line(dst, A, B, Scalar(0, 0, i * 20 + 40), 2, CV_AA);
            line(dst, A, B, Scalar(0, 255, 0), 2, CV_AA);  //gree
        }
        //line(dst, A, B, Scalar(0, 255, 0), 2, CV_AA);  //gree
    }
    showImg("dst", dst);

    //根据角度区分所属指针
    int now_k, pre_k = 720;//当前线段的角度和前一个线段的角度
    int num = 0;//指针数（可能为2或3）
    int Du[10] = { 0 };//3个指针的度数（每组的平均）
    int Le[10] = { 0 };//Le[i]=Le_ping[i]*0.2+le_max[i]*0.8;
    int Le_ping[10] = { 0 };//3个指针的长度（每组的平均）
    int Le_max[10] = { 0 };//3个指针的长度（每组区最大的）
    int t_num = 0;//每组的数量（求平均用）

    MyLine now_Line;
    list_MyLine.push_back(MyLine(99, 888, 0));//向其中插入一个右边界这样方便处理
    list_MyLine.sort();
    while (!list_MyLine.empty())
    {
        now_Line = list_MyLine.front();
        now_k = now_Line.k;
        if (abs(now_k - pre_k)>5)//两个角度之差小于5°的算是同一类
        //if (abs(now_k - pre_k) > 10)//两个角度之差小于10°的算是同一类
        {
            if (num != 0)  //对本组的度数和长度求平均
            {
                Du[num - 1] /= t_num;
                Le_ping[num - 1] /= t_num;
                Le[num - 1] = (int)(Le_ping[num - 1] * 0.2 + Le_max[num - 1] * 0.8);
            }
            if (now_k == 888)  break;//右边界直接跳出
            t_num = 0;//重新统计下一组
            num++; //组数增加1
            if (num > 10) break;
        }
        t_num++;//组内多一条线
        Du[num - 1] += now_Line.k;
        Le_ping[num - 1] += now_Line.l;
        if (now_Line.l > Le_max[num - 1])  Le_max[num - 1] = now_Line.l;
        //now_Line.print();
        //vPrtLogMsg(LOG_DEBUG, RET_OK, "Point id: %d   k: %d°   len: %d", now_Line.id, now_Line.k, now_Line.l);
        //printf("Point id: %d   k: %d°   len: %d\n", now_Line.id, now_Line.k, now_Line.l);
        list_MyLine.pop_front();
        pre_k = now_k;
    }

    int t, duval = 0;
    for (int i = 0; i < num - 1; i++)
    {
        for (int j = i + 1; j < num; j++)
        {
            if (Du[i] <= 0) continue;  // 过滤0°的角
            if (Du[i] < Du[j])
            {
                t = Le[i], Le[i] = Le[j], Le[j] = t;
                t = Du[i], Du[i] = Du[j], Du[j] = t;
            }//if end
        }//for end
    }//for end

    // 根据配置文件的仪表类型，查找指针度数对应的读数

    //printf("\nIn Point id: num:%d  k: %d°   len: %d\n", num, Du[0], Le[0]);

    double dRetVal = stConf->dInitVal;
    double dChaVal = dRetVal;
    for (int n = 0; n < num; n++) {
        val = dGetReadValueOf3ex5050(Du[n], (double)stConf->base_angle, Le[n]);
        if (abs(val - stConf->dInitVal) < dChaVal)
        {
            dRetVal = val;
            dChaVal = abs(val - stConf->dInitVal);
        }
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s Point=%d(%d:%d)  ===  read number is: %.3f", jpg_file, num, Du[n],Le[n],val);
    }
    //if (dRetVal <= 0) dRetVal = 1.0f;
    vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s Point=%d  ===  read number is: %.3f", jpg_file, num, dRetVal);
    return dRetVal;
}

// 读取仪表指针数值
double dReadPointValue_JSY10(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = 0.0;
    Mat temp, dst, gray;                   //src-->temp-->dst(src为原图；temp是src经canny提取边缘的图，
    //用于下面霍夫变换；dst是最终结果图，要在temp灰度图的基础上变为彩色图才能呈现画线效果）
    //rows=行 rols=列

    Mat src = imread(jpg_file, 0);    //读取图片到mat
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    vPrtLogMsg(LOG_DEBUG, RET_OK, "imread picture file %s size:[%dx%d]", jpg_file, src.cols, src.rows);
    //Canny(src, temp, 10, src.cols/4, 3);    //提取边缘(如果不边缘提取就会浪费巨大时间）
    Canny(src, temp, 10, 120, 3);    //提取边缘(如果不边缘提取就会浪费巨大时间）
    cvtColor(temp, dst, CV_GRAY2BGR);//将边缘提取的灰度图转换为BGR图便于画线

    //储存检测圆的容器  
    vector<Vec3f> circles;

    //调用Hough变换检测圆  
    //参数为：待检测图像，检测结果，检测方法（这个参数唯一）,累加器的分辨率，两个圆间的距离，canny门限的上限（下限自动设为上限的一半），圆心所需要的最小的投票数，最大和最小半径  
    //while (!circles.size())
    //HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows/6, 100, 100, 20, 200);
    HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, 50, 200, 100, 100, 300);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 5, 150, 350, 50, 200);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 6, 100, 300, 50, 200);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 7, 80, 240, 50, 200);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 50, 110, 50, 200);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 150, 300, 50, 100);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 100, 200, 40, 80);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 80, 160, 30, 70);
    if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 4, 50, 100, 20, 60);
    if (circles.size() == 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "picture file :%s can't find circles point. return.", jpg_file);
        return -1.0;
    }

    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < temp.rows && f[0] - f[2] >= 0 && f[1] + f[2]<temp.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }

    Point center((int)circles[pos][0], (int)circles[pos][1]);  //找到的圆心
    int   radius = (int)circles[pos][2];                       //找到的半径
    circle(dst, center, radius, Scalar(255, 0, 0), 2);  // BGR=blue
    showImg("Cirles:", dst);
    double vv = 0;
    list<MyLine> list_MyLine;
    vector<Vec4i> lines2;                           //线段检测
    HoughLinesP(temp, lines2, 1, CV_PI / 180, 50, 50, 10);
    //HoughLinesP(temp, lines2, 1, CV_PI / 180, 100, 30, 10);
    if (lines2.size() == 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s can't find line. return.", jpg_file);
        return -1.0;
    }
    vPrtLogMsg(LOG_DEBUG, RET_OK, "file :%s find Cirles=%d,line=%d", jpg_file, (int)circles.size(), (int)lines2.size());
    //    printf("lines2 = %d\n", (int)lines2.size());
    for (size_t i = 0; i < lines2.size(); i++)
    {
        Vec4i l = lines2[i];
        Point A(l[0], l[1]), B(l[2], l[3]);
        vv = DistancetoSegment(center, A, B);
        //if (vv < stConf->max_pointer_len)//根据圆心到指针的距离阈值滤掉其他线段
        if (vv < 30)//根据圆心到指针的距离阈值滤掉其他线段
        {
            bool down = (A.y + B.y - 2 * center.y>0);//判断长的在过圆心的水平线上部还是下部
            if (A.x == B.x)  //斜率为无穷的情况
            {
                list_MyLine.push_back(MyLine(i, 90 + (down ? 180 : 0), (int)Length(Point(A.x - B.x, A.y - B.y))));
            }
            else if (A.y == B.y) //水平的情况
            {
                list_MyLine.push_back(MyLine(i, A.x + B.x - 2 * center.x > 0 ? 0 : 180, (int)Length(Point(A.x - B.x, A.y - B.y))));
            }
            else
            {
                if (down){
                    if (A.y > center.y)
                        list_MyLine.push_back(MyLine(i, 360 - (int)(atan2(A.y - B.y, A.x - B.x) * 180 / PI), (int)Length(Point(A.x - B.x, A.y - B.y))));
                    else
                        list_MyLine.push_back(MyLine(i, 360 - (int)(atan2(B.y - A.y, B.x - A.x) * 180 / PI), (int)Length(Point(A.x - B.x, A.y - B.y))));
                }
                else {
                    if (A.y < center.y)
                        list_MyLine.push_back(MyLine(i, abs((int)(atan2(A.y - B.y, A.x - B.x) * 180 / PI)), (int)Length(Point(A.x - B.x, A.y - B.y))));
                    else
                        list_MyLine.push_back(MyLine(i, abs((int)(atan2(B.y - A.y, B.x - A.x) * 180 / PI)), (int)Length(Point(A.x - B.x, A.y - B.y))));
                }
            }
            //line(dst, A, B, Scalar(0, 0, i * 20 + 40), 2, CV_AA);
            line(dst, A, B, Scalar(0, 255, 0), 2, CV_AA);  //gree
        }
    }
    showImg("dst", dst);

    //根据角度区分所属指针
    int now_k, pre_k = 720;//当前线段的角度和前一个线段的角度
    int num = 0;//指针数（可能为2或3）
    int Du[3] = { 0 };//3个指针的度数（每组的平均）
    int Le[3] = { 0 };//Le[i]=Le_ping[i]*0.2+le_max[i]*0.8;
    int Le_ping[3] = { 0 };//3个指针的长度（每组的平均）
    int Le_max[3] = { 0 };//3个指针的长度（每组区最大的）
    int t_num = 0;//每组的数量（求平均用）

    MyLine now_Line;
    list_MyLine.push_back(MyLine(99, 888, 0));//向其中插入一个右边界这样方便处理
    list_MyLine.sort();
    while (!list_MyLine.empty())
    {
        now_Line = list_MyLine.front();
        now_k = now_Line.k;
        //if (abs(now_k - pre_k)>5)//两个角度之差小于5°的算是同一类
        if (abs(now_k - pre_k) > 10)//两个角度之差小于10°的算是同一类
        {
            if (num != 0)  //对本组的度数和长度求平均
            {
                Du[num - 1] /= t_num;
                Le_ping[num - 1] /= t_num;
                Le[num - 1] = (int)(Le_ping[num - 1] * 0.2 + Le_max[num - 1] * 0.8);
            }
            if (now_k == 888)  break;//右边界直接跳出
            t_num = 0;//重新统计下一组
            num++; //组数增加1
            if (num > 3) break;
        }
        t_num++;//组内多一条线
        Du[num - 1] += now_Line.k;
        Le_ping[num - 1] += now_Line.l;
        if (now_Line.l > Le_max[num - 1])  Le_max[num - 1] = now_Line.l;
        //now_Line.print();
        vPrtLogMsg(LOG_DEBUG, RET_OK, "Point id: %d   k: %d°   len: %d", now_Line.id, now_Line.k, now_Line.l);
        //printf("Point id: %d   k: %d°   len: %d\n", now_Line.id, now_Line.k, now_Line.l);
        list_MyLine.pop_front();
        pre_k = now_k;
    }

    int t, duval = 0;
    for (int i = 0; i < num - 1; i++)
    {
        for (int j = i + 1; j < num; j++)
        {
            //if (Le[i]>Le[j])
            //{
            //  t = Le[i], Le[i] = Le[j], Le[j] = t;
            // t = Du[i], Du[i] = Du[j], Du[j] = t;
            //}//if end
            if (Du[i] < Du[j])
            {
                t = Le[i], Le[i] = Le[j], Le[j] = t;
                t = Du[i], Du[i] = Du[j], Du[j] = t;
            }//if end
        }//for end
    }//for end

    // 根据配置文件的仪表类型，查找指针度数对应的读数

    double dRetVal = stConf->dInitVal;
    double dChaVal = 1;
    for (int n = 0; n < num; n++) {
        val = dGetReadValueOfJSY10(Du[n], (double)stConf->base_angle, Le[n]);
        if (abs(val - stConf->dInitVal) < dChaVal)
        {
            dRetVal = val;
            dChaVal = abs(val - stConf->dInitVal);
        }
    }

    //printf("\nIn Point id: num:%d  k: %d°   len: %d\n", num, Du[0], Le[0]);
    
    vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s Point=%d  ===  read number is: %.3f", jpg_file, num, dRetVal);
    return dRetVal;
}

// 读取仪表指针数值
double dReadPointValue_YZF250TH(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = 0.0;
    Mat temp, dst, gray;                   //src-->temp-->dst(src为原图；temp是src经canny提取边缘的图，
    //用于下面霍夫变换；dst是最终结果图，要在temp灰度图的基础上变为彩色图才能呈现画线效果）
    //rows=行 rols=列

    Mat src = imread(jpg_file, 0);    //读取图片到mat
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    vPrtLogMsg(LOG_DEBUG, RET_OK, "imread picture file %s size:[%dx%d],channel:%d", jpg_file, src.cols, src.rows, src.channels());
    //Canny(src, temp, 20, 80, 3);    //提取边缘(如果不边缘提取就会浪费巨大时间）
    Canny(src, temp, 22, 120, 3);    //提取边缘(如果不边缘提取就会浪费巨大时间）
    cvtColor(temp, dst, CV_GRAY2BGR);//将边缘提取的灰度图转换为BGR图便于画线 CV_GRAY2BGR CV_BGR2GRAY
    
    showImg("HoughCircles TEMP:", src);
    //储存检测圆的容器  
    vector<Vec3f> circles;

    //调用Hough变换检测圆  
    //参数为：待检测图像，检测结果，检测方法（这个参数唯一）,累加器的分辨率，两个圆间的距离，canny门限的上限（下限自动设为上限的一半），圆心所需要的最小的投票数，最大和最小半径  
    HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 1, 50, 200, 100, src.rows / 4, src.rows/2);
    //HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, 50, 200, 50, 100, 200);
    //if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 5, 150, 350, 50, 200);
    //if (circles.size() == 0) HoughCircles(temp, circles, CV_HOUGH_GRADIENT, 2, temp.rows / 6, 100, 300, 50, 200);
    if (circles.size() == 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "picture file :%s can't find circles point. return.", jpg_file);
        return -1.0;
    }
    showImg("HoughCircles TEMP:", temp);
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < temp.rows && f[0] - f[2] >= 0 && f[1] + f[2]<temp.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }

    Point center((int)circles[pos][0], (int)circles[pos][1]);  //找到的圆心
    int   radius = (int)circles[pos][2];                       //找到的半径
    circle(dst, center, radius, Scalar(255, 0, 0), 2);  // BGR=blue
    showImg("HoughCircles dst:", dst);
    double vv = 0;
    list<MyLine> list_MyLine;
    vector<Vec4i> lines2;                           //线段检测
    /*CV_EXPORTS_W void HoughLinesP(InputArray image, OutputArray lines,
        double rho, double theta, int threshold,
        double minLineLength = 0, double maxLineGap = 0);*/
    HoughLinesP(temp, lines2, 1, CV_PI / 180, 50, 50, 10);
    if (lines2.size() == 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s can't find line. return.", jpg_file);
        return -1.0;
    }

    vPrtLogMsg(LOG_DEBUG, RET_OK, "file :%s find Cirles=%d,line=%d", jpg_file, (int)circles.size(), (int)lines2.size());
    for (size_t i = 0; i < lines2.size(); i++)
    {
        Vec4i l = lines2[i];
        Point A(l[0], l[1]), B(l[2], l[3]);

        vv = DistancetoSegment(center, A, B);
        //if (vv < stConf->max_pointer_len)//根据圆心到指针的距离阈值滤掉其他线段
        if (vv < 80)//根据圆心到指针的距离阈值滤掉其他线段
        {
            bool down = (A.y + B.y - 2 * center.y>0);//判断长的在过圆心的水平线上部还是下部
            if (A.x == B.x)  //斜率为无穷的情况
            {
                list_MyLine.push_back(MyLine(i, 90 + (down ? 180 : 0), (int)Length(Point(A.x - B.x, A.y - B.y))));
            }
            else if (A.y == B.y) //水平的情况
            {
                list_MyLine.push_back(MyLine(i, A.x + B.x - 2 * center.x > 0 ? 0 : 180, (int)Length(Point(A.x - B.x, A.y - B.y))));
            }
            else
            {
                if (down){
                    if (A.y > center.y)
                        list_MyLine.push_back(MyLine(i, 360 - (int)(atan2(A.y - B.y, A.x - B.x) * 180 / PI), (int)Length(Point(A.x - B.x, A.y - B.y))));
                    else
                        list_MyLine.push_back(MyLine(i, 360 - (int)(atan2(B.y - A.y, B.x - A.x) * 180 / PI), (int)Length(Point(A.x - B.x, A.y - B.y))));
                }
                else {
                    if (A.y < center.y)
                        list_MyLine.push_back(MyLine(i, abs((int)(atan2(A.y - B.y, A.x - B.x) * 180 / PI)), (int)Length(Point(A.x - B.x, A.y - B.y))));
                    else
                        list_MyLine.push_back(MyLine(i, abs((int)(atan2(B.y - A.y, B.x - A.x) * 180 / PI)), (int)Length(Point(A.x - B.x, A.y - B.y))));
                }
            }
            line(dst, A, B, Scalar(0, 0, i * 20 + 40), 2, CV_AA);
            //line(dst, A, B, Scalar(0, 255, 0), 2, CV_AA);  //gree
        }
    }
    showImg("dst", dst);

    //根据角度区分所属指针
    int now_k, pre_k = 720;//当前线段的角度和前一个线段的角度
    int num = 0;//指针数（可能为2或3）
    int Du[10] = { 0 };//3个指针的度数（每组的平均）
    int Le[10] = { 0 };//Le[i]=Le_ping[i]*0.2+le_max[i]*0.8;
    int Le_ping[10] = { 0 };//3个指针的长度（每组的平均）
    int Le_max[10] = { 0 };//3个指针的长度（每组区最大的）
    int t_num = 0;//每组的数量（求平均用）

    MyLine now_Line;
    list_MyLine.push_back(MyLine(99, 888, 0));//向其中插入一个右边界这样方便处理
    list_MyLine.sort();
    while (!list_MyLine.empty())
    {
        now_Line = list_MyLine.front();
        now_k = now_Line.k;
        //if (abs(now_k - pre_k)>5)//两个角度之差小于5°的算是同一类
        if (abs(now_k - pre_k) > 10)//两个角度之差小于10°的算是同一类
        {
            if (num != 0)  //对本组的度数和长度求平均
            {
                Du[num - 1] /= t_num;
                Le_ping[num - 1] /= t_num;
                Le[num - 1] = (int)(Le_ping[num - 1] * 0.2 + Le_max[num - 1] * 0.8);
            }
            if (now_k == 888)  break;//右边界直接跳出
            t_num = 0;//重新统计下一组
            num++; //组数增加1
            if (num > 10) break;
        }
        t_num++;//组内多一条线
        Du[num - 1] += now_Line.k;
        Le_ping[num - 1] += now_Line.l;
        if (now_Line.l > Le_max[num - 1])  Le_max[num - 1] = now_Line.l;
        //now_Line.print();
        //vPrtLogMsg(LOG_DEBUG, RET_OK, "Point id: %d   k: %d°   len: %d", now_Line.id, now_Line.k, now_Line.l);
        //printf("Point id: %d   k: %d°   len: %d\n", now_Line.id, now_Line.k, now_Line.l);
        list_MyLine.pop_front();
        pre_k = now_k;
    }

    int t, duval = 0;
    for (int i = 0; i<num - 1; i++){
        for (int j = i + 1; j<num; j++){
            if (Le[i]<Le[j]){
                t = Le[i], Le[i] = Le[j], Le[j] = t;
                t = Du[i], Du[i] = Du[j], Du[j] = t;
            }//if end
        }//for end
    }//for end

    // 根据配置文件的仪表类型，查找指针度数对应的读数
    char s[3][10] = { "hour  :", "minute:", "second:" };
    for (int i = 0; i<num; i++)
        printf("%d---  k: %3d°  l: %3d\n", i+1, Du[i], Le[i]);
    cout << "--------------------------->\n";
    //if (num == 2)
    for (int n = 0; n < num ; n++) {
       // val = dGetReadValueOfYZFTH(Du[n], (double)stConf->base_angle, Le[n]);
        //printf("time is: %2d:%2d\n", (360 - Du[n] + 90) % 360 / 30, (360 - Du[n+1] + 90) % 360 / 6);
        printf("=====>> %2d -- %.3f \n", Du[n], val);
    }
    //else if (num == 3)
    //    printf("time is: %2d:%2d:%2d\n", (360 - Du[0] + 90) % 360 / 30, (360 - Du[1] + 90) % 360 / 6, (360 - Du[2] + 90) % 360 / 6);
    //printf("\nIn Point id: num:%d  k: %d°   len: %d\n", num, Du[0], Le[0]);

    double dRetVal = stConf->dInitVal;
    printf("InitVal = %f\n", dRetVal);
    //double dChaVal = dRetVal;
    //for (int n = 0; n < num; n++) {
    //    val = dGetReadValueOfYZFTH(Du[n], (double)stConf->base_angle, Le[n]);
    //    if (abs(val - stConf->dInitVal) < dChaVal)
    //    {
    //        dRetVal = val;
    //        dChaVal = abs(val - stConf->dInitVal);
    //    }
    //    vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s Point=%d(%d:%d)  ===  read number is: %.3f", jpg_file, num, Du[n], Le[n], val);
    //}
    ////if (dRetVal <= 0) dRetVal = 1.0f;
    //vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s Point=%d  ===  read number is: %.3f", jpg_file, num, dRetVal);
    return dRetVal;
}

//double Angle(Point cen, Point first, Point second)
//{
//    const double M_PI = 3.1415926535897;
//
//    double ma_x = first.x - cen.x;
//    double ma_y = first.y - cen.y;
//    double mb_x = second.x - cen.x;
//    double mb_y = second.y - cen.y;
//    double v1 = (ma_x * mb_x) + (ma_y * mb_y);
//    double ma_val = sqrt(ma_x * ma_x + ma_y * ma_y);
//    double mb_val = sqrt(mb_x * mb_x + mb_y * mb_y);
//    double cosM = v1 / (ma_val * mb_val);
//    double angleAMB = acos(cosM) * 180 / M_PI;
//    return angleAMB;
//}
/************************************************************************
*函数名：        get_point_angle
*
*函数作用：      已知2个坐标点，求从 0------->x 逆时针需旋转多少角度到该位置
*
*                   |
*                   |
*                   |
*                   |
*------------------------------------> x
*                   | 0
*                   |
*                   |
*                   |
*                   v
*                   y
*
*函数参数：
*CvPoint2D32f pointO  - 起点
*CvPoint2D32f pointA  - 终点
*
*函数返回值：
*double         向量OA，从 0------->x 逆时针需旋转多少角度到该位置
**************************************************************************/
double get_point_angle(cv::Point pointO, cv::Point pointA)
{
    double angle = 0;
    cv::Point point;
    double temp;

    point = cv::Point((pointA.x - pointO.x), (pointA.y - pointO.y));

    if ((0 == point.x) && (0 == point.y))
    {
        return 0;
    }

    if (0 == point.x)
    {
        angle = 90;
        return angle;
    }

    if (0 == point.y)
    {
        angle = 0;
        return angle;
    }

    temp = fabsf(float(point.y) / float(point.x));
    temp = atan(temp);
    temp = temp * 180 / CV_PI;

    if ((0<point.x) && (0<point.y))
    {
        angle = 360 - temp;
        return angle;
    }

    if ((0>point.x) && (0<point.y))
    {
        angle = 360 - (180 - temp);
        return angle;
    }

    if ((0<point.x) && (0>point.y))
    {
        angle = temp;
        return angle;
    }

    if ((0>point.x) && (0>point.y))
    {
        angle = 180 - temp;
        return angle;
    }

    printf("sceneDrawing :: getAngle error!");
    return -1;
}
// 一号主变冷却器进油温度表
//HNHFT001Q01000106  一号主变冷却器进油温度表 150℃  rtsp://admin:shxy@12345@192.168.1.65:554/cam/realmonitor?channel=1&subtype=0 
double dReadPointValue_Wss411_150_T1(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 38;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));

        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            150, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            100,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            230, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            460  //maxRadius,最大圆半径
            );

        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double  imin = 10.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true)
    {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 120 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        //showImg("line2:", disp);
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    if (p >= 0)
    {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A);
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        // val = (du * 10) / (270);
        val = dGetReadValueOf_WSS150(du, stConf->base_angle, Len);
        if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg("line2:", disp);
    disp.release();
    return val;
}
double dReadPointValue_Wss411_150_T2(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 53;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));

        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            150, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            100,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            230, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            460  //maxRadius,最大圆半径
            );

        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double  imin = 14.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true)
    {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 125 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        //showImg("line2:", disp);
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    if (p >= 0)
    {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A);
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        // val = (du * 10) / (270);
        val = dGetReadValueOf_WSS150(du, stConf->base_angle, Len);
        if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg("line2:", disp);
    disp.release();
    return val;
}
// 3号主变进口油温表分析
double dReadPointValue_Wss411_150_T3(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 50;  //35
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));

        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            200, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            100,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            120, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            320  //maxRadius,最大圆半径
            );

        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double  imin = 10.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true)
    {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 125 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        //showImg("line2:", disp);
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    if (p >= 0)
    {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A);
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        // val = (du * 10) / (270);
        val = dGetReadValueOf_WSS150(du, stConf->base_angle, Len);
        if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg("line2:", disp);
    disp.release();
    return val;
}
// 1#主变出口油压力表1.6MPa
double dReadPointValue_OrcPress_16_T1(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 30;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));
        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            250, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            200,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            220, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            440  //maxRadius,最大圆半径
            );
        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp); 
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double  imin = 10.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true) {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 250 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    showImg(jpg_file, disp);
    if (p >= 0)
    {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A)  + 2.7f;
        //val = (d u * 10) / (248);
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        val = dGetReadValueOfOCR17NI12MO2_16(du, stConf->base_angle, Len);
         if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg(jpg_file, disp);
    disp.release();
    return val;
}
// 1#主变出口油压力表2.5MPa
double dReadPointValue_OrcPress_25_T2(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 77;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));
        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            150, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            100,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            230, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            460  //maxRadius,最大圆半径
            );
        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double  imin = 26.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true) {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 120 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    showImg(jpg_file, disp);
    if (p >= 0)
    {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A) ;
        //val = (du * 10) / (248);
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        val = dGetReadValueOfOCR17NI12MO2_25(du, stConf->base_angle, Len);
        if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg(jpg_file, disp);
    disp.release();
    return val;
}
// 3#主变出口油压力表2.5MPa
double dReadPointValue_OrcPress_25_T3(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 48;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));
        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            200, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            100,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            150, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            320  //maxRadius,最大圆半径
            );
        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double  imin = 10.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true) {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 165 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    showImg(jpg_file, disp);
    if (p >= 0)
    {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A);
        //val = (du * 10) / (248);
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        val = dGetReadValueOfOCR17NI12MO2_25(du, stConf->base_angle, Len);
        if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg(jpg_file, disp);
    disp.release();
    return val;
}
// 3#主变出口油压力表2.5MPa
double dReadPointValue_OrcPress_25_T4(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 48;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));
        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            200, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            100,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            150, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            320  //maxRadius,最大圆半径
            );
        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double  imin = 10.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true) {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 165 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    showImg(jpg_file, disp);
    if (p >= 0)
    {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A);
        //val = (du * 10) / (248);
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        val = dGetReadValueOfOCR17NI12MO2_25(du, stConf->base_angle, Len);
        if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg(jpg_file, disp);
    disp.release();
    return val;
}
// 主变1油表
double dReadPointValue_Wss411_100_T1(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 52;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));
        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            150, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            100,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            220, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            440  //maxRadius,最大圆半径
            );
        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double imin = 10.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true) {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 210 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    if (p >= 0) {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A);// +2.7f;
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        val = dReadPointValue_Wss411_100(du, stConf->base_angle, Len);
        if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg("line2:", disp);
    disp.release();
    return val;
}
// 主变1油表
double dReadPointValue_Wss411_100_T1_2(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 53;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));
        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            150, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            100,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            200, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            440  //maxRadius,最大圆半径
            );
        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double imin = 40.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true) {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 150 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    if (p >= 0) {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A);// +2.7f;
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        val = dReadPointValue_Wss411_100(du, stConf->base_angle, Len);
        if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg("line2:", disp);
    disp.release();
    return val;
}
// 主变1油表
double dReadPointValue_Wss411_100_T2(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 18;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());

    cvtColor(src, src_gray, CV_BGR2GRAY);
    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));
        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            150,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            210, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            200,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            200, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            400  //maxRadius,最大圆半径
            );
        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double imin = 10.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true) {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 190 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    if (p >= 0) {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A);// +2.7f;
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        val = dReadPointValue_Wss411_100(du, stConf->base_angle, Len);
        if (val < 0) val = stConf->dInitVal;
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg("line2:", disp);
    disp.release();
    return val;
}
// 主变2油表
double dReadPointValue_Wss411_100_T3(ST_OPENCV_CONF *stConf, const char *jpg_file)
{
    double val = stConf->dInitVal;
    double threadload = 35;
    vector<Vec3f> circles;
    int radius;
    Mat src_gray, disp, detected_edges;
    Mat src = imread(jpg_file);
    if (!src.data) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "imread picture file :%s failed", jpg_file);
        return -1;
    }
    /// 创建与src同类型和大小的矩阵(dst)
    disp.create(src.size(), src.type());
    cvtColor(src, src_gray, CV_BGR2GRAY);

    while (threadload)
    {
        // 使用 3x3内核降噪
        blur(src_gray, detected_edges, Size(3, 3));
        // 运行Canny算子
        Canny(detected_edges, detected_edges, threadload, threadload * 3, 3);
        disp = Scalar::all(0);
        src.copyTo(disp, detected_edges);   // 将src经过detected_edges掩码(屏蔽)后输出dst.
        HoughCircles(detected_edges, circles, CV_HOUGH_GRADIENT,
            1,   //dp,累加器图像的分辨率,增大则分辨率变小
            100,  //minDist,很重要的一个参数，告诉两个圆之间的距离的最小距离，如果已知一副图像，可以先行计
            //算出符合自己需要的两个圆之间的最小距离。
            220, //param1,canny算法的阈值上限，下限为一半（即100以上为边缘点，50以下抛弃，中间视是否相连而定）
            200,  //param2,决定成圆的多寡 ，一个圆上的像素超过这个阈值，则成圆，否则丢弃
            230, //minRadius,最小圆半径，这个可以通过图片确定你需要的圆的区间范围
            460  //maxRadius,最大圆半径
            );
        if (circles.size() <= 0) {
            threadload--;
            continue;
        }
        break;
    }
    if (circles.size() <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_FAIL, "file :%s,can not found circles.", jpg_file);
        disp.release();  // 释放空间
        return stConf->dInitVal;
    }
    //找出圆盘（因为最大的不一定是的，所以加了几个限制条件）
    int pos = 0;
    int max = -1;
    for (size_t i = 0; i < circles.size(); i++)
    {
        Vec3f f = circles[i];
        if (f[2]>max && f[0] + f[2] < detected_edges.rows && f[0] - f[2] >= 0 && f[1] + f[2]<detected_edges.cols && f[1] - f[2]>0)
        {
            max = (int)f[2];
            pos = i;
        }
    }
    Point center(cvRound(circles[pos][0]), cvRound(circles[pos][1]));
    line(disp, center, center, Scalar(0, 0, 255), 4);   // 画圆心
    radius = cvRound(circles[pos][2]);
    circle(disp, center, radius, Scalar(255, 20, 0), 2);
    showImg("circle:", disp);
    // 找线段
    vector<Vec4i> lines2;
    int Len = 0, p = -1;
    double imin = 10.0f;
    HoughLinesP(detected_edges, lines2, 1, CV_PI / 180, 50, 50, 10);
    while (true) {
        for (int n = 0; n < (int)lines2.size(); n++)
        {
            Point A(lines2[n][0], lines2[n][1]), B(lines2[n][2], lines2[n][3]);  // 一条线段的2头坐标点
            Point C((int)circles[0][0], (int)circles[0][1]);  // 圆心点坐标
            double vv = DistancetoSegment(C, A, B);
            Len = (int)Length(Point(A.x - B.x, A.y - B.y));
            if (Len > 180 && vv < imin) {
                line(disp, A, B, Scalar(0, 255, 0), 2, CV_AA);
                if (vv < imin) {   // 找到离圆心最近的一条线
                    p = n;
                    imin = vv;
                }
            }
        }
        if (p < 0 && imin < radius) {
            imin += 1; continue;
        }
        else break;
    }
    if (p >= 0) {
        Point A(lines2[p][0], lines2[p][1]), B(lines2[p][2], lines2[p][3]);  // 一条线段的2头坐标点
        double du = get_point_angle(B, A) - 2.7f;
        //val = (du * 10) / 205;
        Len = (int)Length(Point(A.x - B.x, A.y - B.y));
        val = dReadPointValue_Wss411_100(du, stConf->base_angle, Len);
        vPrtLogMsg(LOG_DEBUG, RET_OK, "File:%s  Anagle: %.2f°  len: %d  val:%.2f", jpg_file, du, Len, val);
    }
#ifdef _IMG_MIDILL_SAVE
    string files = jpg_file;
    imwrite(files + ".jpg", disp);
#endif
    showImg("line2:", disp);
    disp.release();
    return val;
}
/* 根据配置文件的仪表单位,转为数据库表字段的单位
# -------------------------------------------------------------------
# 仪表单位类型及单位定义对应关系表
# ----------------------------------------
#     类型 | 单位定义 |
# -------------+-------------------------+
# meter_type | meter_units |
# -------------+-------------------+-----+
#              |  0   |  1  |  2   |  3  |
#    = 0       +-------------------+-----+
#   (电流)     | 微安 | 毫安 | 安   |千安 |
#              +-------------------+-----+
#              |  μA  | mA  |  A   |  kA |
#--------------+-------------------+-----+
#    = 1       |  帕  | 百帕 | 千帕 |兆帕 |
#   (压力)     +-------------------+-----+
#              |  Pa  | HPa  | KPa | MPa |
#--------------+-------------------+-----+
#    = 2       | 微升 | 毫升 |  升  |千升 |
#   (容量)     +-------------------+-----+
#              |  μL  | mL   |  L  | kL  |
#--------------+-------------------+-----+
#    = 3       | 摄氏 | 华氏  |     |     |
#   (温度)     +-------------------+-----+
#              |  ℃  |  ℉   |     |     |
#--------------+-------------------+-----+
#    = 4       |   0  |      |     |     |
#   (计数器)   +-------------------+-----+
#              |  次  |      |     |     |
********************************************************************/
double dChangeUnits(unsigned char *sersorid, double inVal)
{
    map<string, ST_OPENCV_CONF>::iterator m_pIter;
    unsigned char meter_type, meter_unit;

    string strKey = (char*)sersorid;
    m_pIter = g_conf_img.find(strKey);

    if (m_pIter == g_conf_img.end())   return inVal; // 没找到配置的单位,则按读到的数据返回即可。
    meter_type = (m_pIter->second.meter_type & 0xff);
    meter_unit = (m_pIter->second.meter_units & 0xff);
    switch (meter_type)
    {
    case 0:  // 电流表,统一为mA
        if (meter_unit == 0)       //微安μA
            inVal = (inVal / 1000);
        else if (meter_unit == 1)
            return inVal; //毫安mA
        else if (meter_unit == 2)  //安A
            inVal = (inVal * 1000);
        else if (meter_unit == 3)  //千安kA
            inVal = (inVal * 1000 * 1000);
        break;
    case 1:  // 压力表,统一为MPa
        if (meter_unit == 0) //帕Pa
            inVal = (inVal / 1000 / 1000);
        else if (meter_unit == 1) //百帕HPa
            inVal = (inVal / 1000 / 100);
        else if (meter_unit == 2) //千帕KPa
            inVal = (inVal / 1000);
        else if (meter_unit == 3) //兆帕MPa
            return inVal;
        break;
    case 2:  // 容量表/油位表...,0：缺油 1：正常，两种状态即可
        break;
    case 3:  // 温度表,统一为摄氏度°
        if (meter_unit == 0) //度℃
            return inVal;
        if (meter_unit == 1) //华氏度
            inVal = (inVal - 32 / 1.800f);
        break;
    case 4:  // 计数器,放电次数
        break;
    default:; // 未知表类型
    }
    return inVal;
}

// 获取sersorid对应的告警阈值
//
//static double dGetWarningValue(const char *sersorid, ST_BLQ_CACHE &stBlq)
//{
//    if (sersorid == NULL || strlen(sersorid) <=0) return 0.0f;
//
//    string strKey = sersorid;
//    map<string, ST_BLQ_CACHE>::iterator m_pIter;
//    mutex_lock(g_map_blq_mutex);
//    m_pIter = g_map_blq.find(strKey);
//    if(m_pIter == g_map_blq.end())
//    {
//        mutex_unlock(g_map_blq_mutex);
//        vPrtLogMsg(LOG_WARNG, RET_FAIL, "Can't find sersorid:%s for set warning value.",sersorid);
//        return 0.0f;
//    }
//    memcpy(&stBlq, &(m_pIter->second), sizeof(ST_BLQ_CACHE));
//    mutex_unlock(g_map_blq_mutex);
//    return stBlq.dMaxDL;
//}

// 告警数据入库
static void dbInsertImgageWarning(ST_IMG_THRESHOLD &stCache, double dval, double threshold, unsigned char wstate, unsigned char eqm_type, char *pWarnTime)
{
    char szSql[DEF_BUFFER_1K] = { 0 }, szFormatTime[32] = { 0 };
    char szUuid[64] = { 0 };
    char szWarninfo[DEF_BUFFER_256] = { 0 };

    vFormatTimes(pWarnTime, szFormatTime);  // yyyymmddhhmiss-->yyyy-mm-dd hh24:mi:ss
    CDBMySQL *pdbHandle = CDBMySQL::Instance();
    if (!pdbHandle) return;
    
    getuuid(szUuid, sizeof(szUuid));

    snprintf(szSql, sizeof(szSql),"INSERT INTO busi_eqm_warning(id,eq_type,gz_type,warning_msg,warning_time,create_time,del_flag,site_id, main_tree_id, sensor_id,run_state) "
        "VALUES('%s','%d', '11', '%s:%.4f', '%s', SYSDATE(), '1', '%s', '%s', '%s','%d')", szUuid,eqm_type,stCache.desc, dval, pWarnTime, stCache.site_id, stCache.eqm_code, stCache.sensor_id,wstate);
    vPrtLogMsg(LOG_DEBUG, RET_OK, "dbInsertWaring: %s", szSql);
    if (!pdbHandle->InsertRecord((const char *)szSql)) {
        vPrtLogMsg(LOG_ERROR, RET_FAIL, "INSERT INTO busi_eqm_warning message failed,SQL:%s", szSql);
        return;
    }
    snprintf(szWarninfo, sizeof(szWarninfo),
        "{\"sadr\":\"\",\"thresholdValue\":\"%.4f\",\"currRealValue\":\"%.4f\",\"equipmentStatus\":\"%d\",\"collectDate\":\"%s\",\"ipaddr\":\"\",\"port\":\"\",\"msgInfo\":\"%s\",\"phase\":\"%c\"}",
        threshold, dval, wstate, szFormatTime, stCache.desc, stCache.phase);
    snprintf(szSql, sizeof(szSql), "INSERT INTO busi_eqm_warning_info(id,warning_id,warning_info) VALUES(guuid(),'%s','%s')", szUuid, szWarninfo);

    vPrtLogMsg(LOG_DEBUG, RET_OK, "dbInsertWaring: %s", szSql);
    if (!pdbHandle->InsertRecord((const char *)szSql)) {
        vPrtLogMsg(LOG_ERROR, RET_FAIL, "INSERT INTO busi_eqm_warning message failed,SQL:%s", szSql);
        return;
    }
}

//BLQ入库仪表解析数据
static void dbBLQInsertData(ST_IMG_THRESHOLD &stBlqCache, double dval, const char *filepath, char *pTime, int wstate)
{
    char szSql[DEF_BUFFER_1K] = { 0 };
    char szHbaseKey[DEF_BUFFER_128] = { 0 }; // hbase同步key.
    char szSortTime[24] = { 0 };

    memcpy(szSortTime, pTime, strlen(pTime));
    strSwap(szSortTime, strlen(szSortTime));

    CDBMySQL *pdbHandle = CDBMySQL::Instance();
    if (!pdbHandle) return;

    sprintf(szHbaseKey, "%s_%s_%s_%s", szSortTime, stBlqCache.sensor_id, pTime, stBlqCache.eqm_code);
    sprintf(szSql, "INSERT INTO arrester_read_table(id, site_id, system_code, sensor_code,dlval,time_stamp,work_state,main_tree_id,picpath,hbase_rowkey) "
        "VALUES(guuid(),'%s', '%s', '%s', '%.4f','%s','%d','%s','%s','%s')", stBlqCache.site_id, stBlqCache.system_code, stBlqCache.sensor_id, 
        dval, pTime, wstate, stBlqCache.eqm_code, filepath, szHbaseKey);

    vPrtLogMsg(LOG_DEBUG, RET_OK, "dbBLQInsertData: %s", szSql);
    if (!pdbHandle->InsertRecord((const char *)szSql)) {
        vPrtLogMsg(LOG_ERROR, RET_FAIL, "INSERT INTO arrester_read_table failed,SQL:%s", szSql);
        return;
    }
}

//入库仪表解析数据
static void dbGISInsertData(ST_IMG_THRESHOLD &stGisCache, double dval, const char *filepath, char *pTime, int wstate)
{
    char szSql[DEF_BUFFER_1K] = { 0 };
    char szHbaseKey[DEF_BUFFER_128] = { 0 }; // hbase同步key.
    char szSortTime[24] = { 0 };

    memcpy(szSortTime, pTime, strlen(pTime));
    strSwap(szSortTime, strlen(szSortTime));

    CDBMySQL *pdbHandle = CDBMySQL::Instance();
    if (!pdbHandle) return;

    sprintf(szHbaseKey, "%s_%s_%s_%s", szSortTime, stGisCache.sensor_id, pTime, stGisCache.eqm_code);
    sprintf(szSql, "INSERT INTO gis_read_table(id,site_id, system_code, sensor_code,time_stamp,dataval,picpath,main_tree_id,work_state,hbase_rowkey) "
        "VALUES(guuid(),'%s', '%s', '%s','%s', '%.4f','%s','%s','%d','%s')", stGisCache.site_id, stGisCache.system_code,
        stGisCache.sensor_id, pTime, dval, filepath, stGisCache.eqm_code,wstate,szHbaseKey);
    vPrtLogMsg(LOG_DEBUG, RET_OK, "dbGISInsertData: %s", szSql);
    if (!pdbHandle->InsertRecord((const char *)szSql)) {
        vPrtLogMsg(LOG_ERROR, RET_FAIL, "INSERT INTO gis_read_table failed,SQL:%s", szSql);
        return;
    }
}
//入库仪表解析数据
static void dbBYQInsertData(ST_IMG_THRESHOLD &stCache, double dval, const char *filepath, char *pTime, int wstate)
{
    // 数据入库
    char szSql[DEF_BUFFER_1K] = { 0 }, szTime[21] = { 0 };
    char szHbaseKey[DEF_BUFFER_128] = { 0 }; // hbase同步key.
    char szSortTime[24] = { 0 };

    memcpy(szSortTime, pTime, strlen(pTime));
    strSwap(szSortTime, strlen(szSortTime));

    CDBMySQL *pdbHandle = CDBMySQL::Instance();
    if (!pdbHandle) {
        mutex_unlock(g_map_byq_mutex);
        return;
    }
    // 原始数据入库
    sprintf(szHbaseKey, "%s_%s_%s_%s", szSortTime, stCache.sensor_id, pTime, stCache.eqm_code);
    memset(szSql, 0x00, sizeof(szSql));
    sprintf(szSql, "INSERT INTO transformer_read_table(id,site_id,main_tree_id,sensor_code,system_code,d_time,dataval,work_state,picpath,hbase_rowkey) "
        "VALUES(guuid(),'%s','%s', '%s', '%s','%s','%.4f', '%d', '%s','%s')",
        stCache.site_id, stCache.eqm_code, stCache.sensor_id, stCache.system_code, pTime, dval, wstate, filepath, szHbaseKey);
    vPrtLogMsg(LOG_DEBUG, RET_OK, "transformer_read_table: %s", szSql);
    if (!pdbHandle->InsertRecord((const char *)szSql)) {
        vPrtLogMsg(LOG_ERROR, RET_FAIL, "INSERT INTO transformer_read_table message failed,SQL:%s", szSql);
    }
}

// 在一段时间内，统计GIS放电的次数
//static int iGetGisFangDianCount(ST_GIS_CACHE &stGis)
//{
//    int count = 0;
//    char szSql[1024] = { 0 }, sTime[24] = { 0 }, eTime[24] = { 0 };
//
//    // 取一天的起始和结束时间
//    vGeOneDayTimeFmt(sTime, eTime);
//    CDBMySQL *pdbHandle = CDBMySQL::Instance();
//    if (!pdbHandle) return 0;
//#ifdef _SITE_ID_TYPE_INT
//    sprintf(szSql, "SELECT SUM(fdcs) FROM gis_read_table WHERE site_id='%u' AND sersorid='%s' AND eqm_code='%s' "
//        "AND DATE(d_time) BETWEEN '%s' AND '%s'", stGis.SiteID, (char*)stGis.szSersorID, (char*)stGis.szEqmCodeID, sTime, eTime);
//#else
//    sprintf(szSql, "SELECT SUM(fdcs) FROM gis_read_table WHERE site_id='%s' AND sersorid='%s' AND eqm_code='%s' "
//        "AND DATE(d_time) BETWEEN '%s' AND '%s'", stGis.SiteID, (char*)stGis.szSersorID, (char*)stGis.szEqmCodeID, sTime, eTime);
//#endif
//    vPrtLogMsg(LOG_DEBUG, RET_OK, "iGetGisFangDianCount:SQL[%s]", szSql);
//    MYSQL_RES *res = pdbHandle->SelectRecord(szSql);
//    if (!res)  return 0;
//
//    MYSQL_ROW row = NULL;
//    if (row = pdbHandle->GetRecord(res))
//    {
//        (row[0] && strlen(row[0])) ? (count = atoi((const char*)row[0])) : 0;
//    }
//    pdbHandle->FreeRecord(res);
//
//    return count;
//}


// 是否存在告警
unsigned char isImageWaring(ST_OPENCV_CONF *stConf, double dValue, char *pszCreateTime, ST_IMG_THRESHOLD &stThresdhold)
{
    // 检查是否有阈值告警
    // 获取工作状态
    unsigned char ws = cGetCurrentWorkState(stConf->eqm_code);
    if (ws <= 0) {
        vPrtLogMsg(LOG_WARNG, RET_OK, "get device:%s status, can not found bind relations ...", (char*)stConf->eqm_code);
        strcpy((char*)stThresdhold.site_id, (const char*)stConf->site_id);
        strcpy((char*)stThresdhold.eqm_code, (const char*)stConf->eqm_code);
        strcpy((char*)stThresdhold.system_code, (const char*)stConf->sys_code);
        strcpy((char*)stThresdhold.sensor_id, (const char*)stConf->sersorid);
        return ws;  // 默认返回空闲状态
    }
    unsigned char eqm_type = 0;
    //获取该状态下的阈值
    if (getImageThresholdValuse((const char*)stConf->sersorid, stThresdhold))
    {
        if (stThresdhold.eqm_type == 1)  eqm_type = 1; // 避雷器
        else if (stThresdhold.eqm_type == 2)  eqm_type = 2; // GIS
        else if (stThresdhold.eqm_type == 3)  eqm_type = 4; // 测温
        else if (stThresdhold.eqm_type == 4)  eqm_type = 3; // 避雷器
        //判断是否告警
        switch (ws)
        {
        case 1:
            if (stConf->meter_type == 1){  // 压力时,小于阈值报警
                if (dValue < stThresdhold.value[0] && stThresdhold.value[0] != HT_INVALID_VALUE)
                    dbInsertImgageWarning(stThresdhold, dValue, stThresdhold.value[0], ws, eqm_type, pszCreateTime);
            }
            else if (dValue > stThresdhold.value[0] && stThresdhold.value[0] != HT_INVALID_VALUE)
                dbInsertImgageWarning(stThresdhold, dValue, stThresdhold.value[0], ws, eqm_type, pszCreateTime);
            break;
        case 2:
            if (stConf->meter_type == 1){  // 压力时,小于阈值报警
                if (dValue < stThresdhold.value[1] && stThresdhold.value[1] != HT_INVALID_VALUE)
                    dbInsertImgageWarning(stThresdhold, dValue, stThresdhold.value[1], ws, eqm_type, pszCreateTime);
            }
            else if (dValue > stThresdhold.value[1] && stThresdhold.value[1] != HT_INVALID_VALUE)
                dbInsertImgageWarning(stThresdhold, dValue, stThresdhold.value[1], ws, eqm_type, pszCreateTime);
            break;
        case 3:
            if (stConf->meter_type == 1){  // 压力时,小于阈值报警
                if (dValue < stThresdhold.value[2] && stThresdhold.value[2] != HT_INVALID_VALUE)
                    dbInsertImgageWarning(stThresdhold, dValue, stThresdhold.value[2], ws, eqm_type, pszCreateTime);
            }
            else if (dValue > stThresdhold.value[2] && stThresdhold.value[2] != HT_INVALID_VALUE)
                dbInsertImgageWarning(stThresdhold, dValue, stThresdhold.value[2], ws, eqm_type, pszCreateTime);
            break;
        default:;
        }
    }
    else {
        vPrtLogMsg(LOG_WARNG, RET_OK, "get device:%s sensorid:%s thresdhold, can not found setting threldhold!", (char*)stConf->eqm_code, (char*)stConf->sersorid);
        strcpy((char*)stThresdhold.site_id, (const char*)stConf->site_id);
        strcpy((char*)stThresdhold.eqm_code, (const char*)stConf->eqm_code);
        strcpy((char*)stThresdhold.system_code, (const char*)stConf->sys_code);
        strcpy((char*)stThresdhold.sensor_id, (const char*)stConf->sersorid);
    }
    return ws;
}
// 分析避雷器图片
static void vBLQParserPicture(ST_OPENCV_CONF *stConf)
{
    char szPath[MAX_PATH] = { 0 }, szFile[MAX_PATH] = { 0 }, szExt[8] = { 0 }, szCreateTime[24] = { 0 };
    int  filecount = 0;
    double dWarnVal = 0.0;
    ST_IMG_THRESHOLD stThresdhold;
    std::vector<cv::String> vecFileList;
    double dValue = 0.0;  // 指针读数   

    filecount = iGetImgFileList((const char*)stConf->img_path, vecFileList); // 扫描文件列表
    // 逐个文件分析
    for (int n = 0; n < filecount; n++)
    {
        // 目录转为Linux格式
        Window2UnixPath((char*)vecFileList[n].c_str());

        // 必须在截图之前，获取文件的创建时间，作为告警发生时间。
        if (getFileCreateTime((char*)vecFileList[n].c_str(), szCreateTime) < stConf->img_min_size)
        {
            vPrtLogMsg(LOG_WARNG, RET_OK, "file:%s size not enjon,process next file...", (char*)vecFileList[n].c_str());
            continue;   // 获取原文件创建时间
        }
        vPrtLogMsg(LOG_DEBUG, RET_OK, "img_file: %s ctime:%s", vecFileList[n].c_str(), szCreateTime);

#ifdef _HT_OPENCV_TEST_ENV
        dValue = getRandByTime(); // for test
        //vGetHostTimeFmt(szCreateTime);
        vGetHostTime(szCreateTime);
        //vGetHostTimeFmtBeforBay(szCreateTime, 1);
#else
        // 截图处理
        vCutImageSave(stConf, (char*)vecFileList[n].c_str());

        // 分析图片
        dValue = dReadPointValue_3EX5050(stConf, vecFileList[n].c_str());
        if (dValue < 0) continue;
#endif     

        // 单位转换，转为与数据库表字段相同的单位
        dValue = dChangeUnits(stConf->sersorid, dValue);

        // 检查是否有阈值告警
        memset(&stThresdhold, 0x00, sizeof(ST_IMG_THRESHOLD));
        unsigned char ws = isImageWaring(stConf, dValue, szCreateTime, stThresdhold);
        // 获取工作状态
#ifndef _HT_OPENCV_TEST_ENV
        // 备份文件,分析完成的图片移到备份目录下.
        bBakFilename((char*)vecFileList[n].c_str(), (char*)stConf->img_bak);
#endif
        // 分解出目录、文件、及扩展名
        bSplitFilenameByFullPath((char*)vecFileList[n].c_str(), szPath, szFile, szExt);

        // 备份后的文件路径及文件名
        sprintf(szPath, "%s/%s.%s", stConf->img_bak, szFile, szExt);

        // 数据入库
        dbBLQInsertData(stThresdhold, dValue, (const char*)szPath, szCreateTime, ws);
    }
    return;
}


// 分析GIS图片,GIS没有放电次数的阈值报警,只有放电电流阈值报警了。
static void vGISParserPicture(ST_OPENCV_CONF *stConf)
{
    char szPath[MAX_PATH] = { 0 }, szFile[MAX_PATH] = { 0 }, szExt[8] = { 0 }, szCreateTime[24] = { 0 };
    char szSrcFileName[MAX_PATH] = { 0 };
    int  filecount = 0, iType = 0;
    ST_IMG_THRESHOLD stThresdhold;
    vector<cv::String> vecFileList;
    double dValue = 0;  // 指针读数   

    filecount = iGetImgFileList((const char*)stConf->img_path, vecFileList); // 扫描文件列表

    // 逐个文件分析
    for (int n = 0; n < filecount; n++)
    {
        // 目录转为Linux格式 
        memset(szSrcFileName, 0x00, sizeof(szSrcFileName));
        memset(szCreateTime, 0x00, sizeof(szCreateTime));
        strncpy(szSrcFileName, vecFileList[n].c_str(), vecFileList[n].length());
        Window2UnixPath(szSrcFileName);

        // 必须在截图之前，获取文件的创建时间，作为告警发生时间。
        if (getFileCreateTime(szSrcFileName, szCreateTime) < stConf->img_min_size) {
            vPrtLogMsg(LOG_WARNG, RET_OK, "file:%s size not enjon,process next file...", szSrcFileName);
            continue;   // 获取原文件创建时间,返回值小于图片文件最大字节时，说明本文件尚未完整保存。
        }
#ifdef _HT_OPENCV_TEST_ENV
        dValue = getRandByTime(); // for test
        //vGetHostTimeFmt(szCreateTime);
        vGetHostTime(szCreateTime);
        //vGetHostTimeFmtBeforBay(szCreateTime, 1);
#else
        // 截图处理
        vCutImageSave(stConf, szSrcFileName);

        // 分析图片
        dValue = dReadPointValue_JSY10(stConf, szSrcFileName);
        if (dValue < 0) continue;
#endif
        // 单位转换，转为与数据库表字段相同的单位
        dValue = dChangeUnits(stConf->sersorid, dValue);

        // 检查是否有放电电流阈值告警, 当前值大于预警值,则插入告警信息
        memset(&stThresdhold, 0x00, sizeof(ST_IMG_THRESHOLD));
        unsigned char ws = isImageWaring(stConf, dValue, szCreateTime, stThresdhold);
#ifndef _HT_OPENCV_TEST_ENV
        // 备份文件,分析完成的图片移到备份目录下.
        bBakFilename(szSrcFileName, (char*)stConf->img_bak);
#endif
        // 分解出目录、文件、及扩展名
        bSplitFilenameByFullPath(szSrcFileName, szPath, szFile, szExt);

        // 备份后的文件路径及文件名
        sprintf(szPath, "%s/%s.%s", stConf->img_bak, szFile, szExt);

        // 数据入库
        dbGISInsertData(stThresdhold, dValue, (const char*)szPath, szCreateTime, ws); 
    }
}

// 分析BYQ油位图片。
static void vBYQParserPicture(ST_OPENCV_CONF *stConf)
{
    char szPath[MAX_PATH] = { 0 }, szFile[MAX_PATH] = { 0 }, szExt[8] = { 0 }, szCreateTime[24] = { 0 };
    char szSrcFileName[MAX_PATH] = { 0 };
    int  filecount = 0, iType = 0;
    ST_IMG_THRESHOLD stThresdhold;
    vector<cv::String> vecFileList;
    double dValue = 0;  // 指针读数   

    filecount = iGetImgFileList((const char*)stConf->img_path, vecFileList); // 扫描文件列表

    // 逐个文件分析
    for (int n = 0; n < filecount; n++)
    {
        // 目录转为Linux格式 
        memset(szSrcFileName, 0x00, sizeof(szSrcFileName));
        memset(szCreateTime, 0x00, sizeof(szCreateTime));
        strncpy(szSrcFileName, vecFileList[n].c_str(), vecFileList[n].length());
        Window2UnixPath(szSrcFileName);

        // 必须在截图之前，获取文件的创建时间，作为告警发生时间。
        if (getFileCreateTime(szSrcFileName, szCreateTime) < stConf->img_min_size) {
            vPrtLogMsg(LOG_WARNG, RET_OK, "file:%s size not enjon,process next file...", szSrcFileName);
            continue;   // 获取原文件创建时间,返回值小于图片文件最大字节时，说明本文件尚未完整保存。
        }
#ifdef _HT_OPENCV_TEST_ENV
        dValue = getRandByTime(); // for test
        //vGetHostTimeFmt(szCreateTime);
        vGetHostTime(szCreateTime);
        //vGetHostTimeFmtBeforBay(szCreateTime,1);
#else
        // 截图处理
        vCutImageSave(stConf, szSrcFileName);
        // 分析图片
        if (stringncasecmp((const char*)stConf->sys_code, (const char*)"5GST1QLZJ6ORXMK0U", 17) == 0)
        {   // 1#主变图片分析
            if (stringncasecmp((const char*)stConf->table_type, (const char*)"WSS-411-150", 11) == 0)
                dValue = dReadPointValue_Wss411_150_T1(stConf, szSrcFileName);   // #1进口油温仪表分析
            else if (stringncasecmp((const char*)stConf->table_type, (const char*)"OCR17NI12MO2-16", 15) == 0)
                dValue = dReadPointValue_OrcPress_16_T1(stConf, szSrcFileName);   // #1主变出水口压力仪表分析
            else if (stringncasecmp((const char*)stConf->table_type, (const char*)"WSS-411-100-1", 13) == 0)
                dValue = dReadPointValue_Wss411_100_T1(stConf, szSrcFileName);   // #1进出口水温仪表分析
            else if (stringncasecmp((const char*)stConf->table_type, (const char*)"WSS-411-100-2", 13) == 0)
                dValue = dReadPointValue_Wss411_100_T1_2(stConf, szSrcFileName);   // #1进出口水温仪表分析
            else  {
                vPrtLogMsg(LOG_WARNG, RET_OK, "table_type=[%s] not find devices type,pls check [htimg.conf]", stConf->table_type);
                return;
            }
        }
        else if (stringncasecmp((const char*)stConf->sys_code, "BOLBJMZFOYMSMCUIO", 17) == 0) //||
        {  // 2#主变图片分析
            if (stringncasecmp((const char*)stConf->table_type, (const char*)"WSS-411-150", 11) == 0)
                dValue = dReadPointValue_Wss411_150_T2(stConf, szSrcFileName);   // #2进口油温仪表分析
            else if (stringncasecmp((const char*)stConf->table_type, (const char*)"OCR17NI12MO2-25", 15) == 0)
                dValue = dReadPointValue_OrcPress_25_T2(stConf, szSrcFileName);   // #2主变冷却出水压力
            else if (stringncasecmp((const char*)stConf->table_type, (const char*)"WSS-411-100", 11) == 0)
                dValue = dReadPointValue_Wss411_100_T2(stConf, szSrcFileName);   // #2进出口水温仪表分析
            else  {
                vPrtLogMsg(LOG_WARNG, RET_OK, "table_type=[%s] not find devices type,pls check [htimg.conf]", stConf->table_type);
                return;
            }
        }
        else if (stringncasecmp((const char*)stConf->sys_code, "YYKL3BWFOWB0K8H2K", 17) == 0)
        {   // 3#主变图片分析
            if (stringncasecmp((const char*)stConf->table_type, (const char*)"WSS-411-150", 11) == 0)
                dValue = dReadPointValue_Wss411_150_T3(stConf, szSrcFileName);   // #3进口油温仪表分析
            else if (stringncasecmp((const char*)stConf->table_type, (const char*)"OCR17NI12MO2-25", 15) == 0)
                dValue = dReadPointValue_OrcPress_25_T3(stConf, szSrcFileName);   // #3主变冷却出水压力
            else if (stringncasecmp((const char*)stConf->table_type, (const char*)"WSS-411-100", 11) == 0)
                dValue = dReadPointValue_Wss411_100_T3(stConf, szSrcFileName);   // #3进出口水温仪表分析
            else  {
                vPrtLogMsg(LOG_WARNG, RET_OK, "table_type=[%s] not find devices type,pls check [htimg.conf]", stConf->table_type);
                return;
            }
        }
        else if (stringncasecmp((const char*)stConf->sys_code, "1PH197OOZMW7DTMS8", 17) == 0)
        {   // 4#主变图片分析
            if (stringncasecmp((const char*)stConf->table_type, (const char*)"WSS-411-150", 11) == 0)
                dValue = dReadPointValue_Wss411_150_T1(stConf, szSrcFileName);   // #4进口油温仪表分析
            else if (stringncasecmp((const char*)stConf->table_type, (const char*)"OCR17NI12MO2-25", 15) == 0)
                dValue = dReadPointValue_OrcPress_25_T4(stConf, szSrcFileName);   // #4主变冷却出水压力
            else if (stringncasecmp((const char*)stConf->table_type, (const char*)"WSS-411-100", 11) == 0)
                dValue = dReadPointValue_Wss411_100_T1(stConf, szSrcFileName);   // #4进出口水温仪表分析
            else  {
                vPrtLogMsg(LOG_WARNG, RET_OK, "table_type=[%s] not find devices type,pls check [htimg.conf]", stConf->table_type);
                return;
            }
        }
        else {
            vPrtLogMsg(LOG_WARNG, RET_OK, "sys_code=[%s] not find devices code,pls check [htimg.conf]", stConf->eqm_code);
            return;
        }
        if (dValue < 0) continue;
#endif
        // 单位转换，转为与数据库表字段相同的单位
        dValue = dChangeUnits(stConf->sersorid, dValue);

        // 检查是否有阈值告警, 当前值大于预警值,则插入告警信息
        memset(&stThresdhold, 0x00, sizeof(ST_IMG_THRESHOLD));
        unsigned char ws = isImageWaring(stConf, dValue, szCreateTime, stThresdhold);
#ifndef _HT_OPENCV_TEST_ENV
        // 备份文件,分析完成的图片移到备份目录下.
        bBakFilename(szSrcFileName, (char*)stConf->img_bak);
#endif
        // 分解出目录、文件、及扩展名
        bSplitFilenameByFullPath(szSrcFileName, szPath, szFile, szExt);

        // 备份后的文件路径及文件名
        snprintf(szPath,sizeof(szPath), "%s/%s.%s", stConf->img_bak, szFile, szExt);

        // 数据入库
        dbBYQInsertData(stThresdhold, dValue, (const char*)szPath, szCreateTime, ws);
        _SLEEP(100);
    }
}

/***************************************************************************
** function name :  thread_opencv_proc
** deacription   :  opencv img parser thread
** parameter     :  none
** return code   :  NULL
***************************************************************************/
void* thread_opencv_proc(void * arg)
{
    ST_OPENCV_CONF stConf;
    map<string, ST_OPENCV_CONF>::iterator m_pIter;
    char szCmd[512] = { 0 };

    vPrtLogMsg(LOG_DEBUG, 0, "thread_opencv_proc = %d startup...", GETTID());
    // 获取配置文件数据
    if (!iGetOpencvConf()) {
        vPrtLogMsg(LOG_ERROR, errno, "get opencv config failed, thread_opecv_proc exit.");
        return NULL;
    }

    vPrtLogMsg(LOG_DEBUG, RET_OK, "get opencv config complating...size:%d", g_conf_img.size());
    _SLEEP(500);
    while (g_Running && g_conf_img.size())
    {
        // 扫描图片目录,获取识别的图片文件列表
        pthread_testcancels();

        for (m_pIter = g_conf_img.begin(); m_pIter != g_conf_img.end(); m_pIter++)
        {
            memset(&stConf, 0x00, sizeof(ST_OPENCV_CONF));
            memcpy(&stConf, &(m_pIter->second), sizeof(ST_OPENCV_CONF));
            vPrtLogMsg(LOG_DEBUG, 0, "--> No=%d BEGIN sersorid: %s file List.", stConf.seqno, stConf.sersorid);
            if ((stConf.sersorid[4] & 0xff) == 'T' || stConf.eqm_type[0] == 'T' ) {  // BYQ图片处理,油位表读数
                vBYQParserPicture(&stConf);
            }
            
            if ((stConf.sersorid[4] & 0xff) == 'G' || stConf.eqm_type[0] == 'G') {  // GIS图片处理,放电次数
                vGISParserPicture(&stConf);
            }

            if ((stConf.sersorid[4] & 0xff) == 'B' || stConf.eqm_type[0] == 'B') {  // 避雷器图片处理,放电电流
                vBLQParserPicture(&stConf);
            }
            vPrtLogMsg(LOG_DEBUG, 0, "--> No=%d ENDED sersorid: %s file List.", stConf.seqno, stConf.sersorid);
            //_SLEEP(1000 * 5);
#ifndef _HT_OPENCV_TEST_ENV
            // 备份文件,分析完成的图片移到备份目录下.
            //bBakFilename(szSrcFileName, (char*)stConf->img_bak);
            memset(szCmd, 0x00, sizeof(szCmd));
#ifdef _WIN32
            sprintf(szCmd, "move %s ../spic/", (char*)stConf.img_path);
#else
            sprintf(szCmd, "mv %s /appdata/sftp/spic/", (char*)stConf.img_path);
#endif
            system(szCmd);
#endif
        }
        _SLEEP(1000 * 60);
    }
    vPrtLogMsg(LOG_DEBUG, RET_OK, "link exceptiond, thread_opencv_proc pthread_exit.");
    return NULL;
}

